@@ -18,11 +18,12 @@ images = ('astronaut',
           'brick',
           'colorwheel',
           'camera',
+          'cat',
           'checkerboard',
-          'chelsea',
           'clock',
           'coffee',
           'coins',
+          'eagle',
           'grass',
           'gravel',
           'horse',
