@@ -7,7 +7,7 @@ The Hough transform in its simplest form is a method to detect straight lines
 [1]_.
 
 In the following example, we construct an image with a line intersection. We
-then use the `Hough transform  <http://en.wikipedia.org/wiki/Hough_transform>`__.
+then use the `Hough transform  <https://en.wikipedia.org/wiki/Hough_transform>`__.
 to explore a parameter space for straight lines that may run through the image.
 
 Algorithm overview
@@ -23,9 +23,10 @@ the length of that segment, :math:`r`, and the angle it makes with the x-axis,
 The Hough transform constructs a histogram array representing the parameter
 space (i.e., an :math:`M \\times N` matrix, for :math:`M` different values of
 the radius and :math:`N` different values of :math:`\\theta`).  For each
-parameter combination, :math:`r` and :math:`\\theta`, we then find the number of
-non-zero pixels in the input image that would fall close to the corresponding
-line, and increment the array at position :math:`(r, \\theta)` appropriately.
+parameter combination, :math:`r` and :math:`\\theta`, we then find the number
+of non-zero pixels in the input image that would fall close to the
+corresponding line, and increment the array at position :math:`(r, \\theta)`
+appropriately.
 
 We can think of each non-zero pixel "voting" for potential line candidates. The
 local maxima in the resulting histogram indicates the parameters of the most
@@ -55,10 +56,14 @@ References
        Conference on Computer Vision and Pattern Recognition, 1999.
 
 """
+
+######################
+# Line Hough Transform
+# ====================
+
 import numpy as np
 
-from skimage.transform import (hough_line, hough_line_peaks,
-                               probabilistic_hough_line)
+from skimage.transform import hough_line, hough_line_peaks
 from skimage.feature import canny
 from skimage import data
 
@@ -67,13 +72,15 @@ from matplotlib import cm
 
 
 # Constructing test image
-image = np.zeros((100, 100))
-idx = np.arange(25, 75)
+image = np.zeros((200, 200))
+idx = np.arange(25, 175)
 image[idx[::-1], idx] = 255
 image[idx, idx] = 255
 
 # Classic straight-line Hough transform
-h, theta, d = hough_line(image)
+# Set a precision of 0.5 degree.
+tested_angles = np.linspace(-np.pi / 2, np.pi / 2, 360)
+h, theta, d = hough_line(image, theta=tested_angles)
 
 # Generating figure 1
 fig, axes = plt.subplots(1, 3, figsize=(15, 6))
@@ -92,11 +99,11 @@ ax[1].set_ylabel('Distance (pixels)')
 ax[1].axis('image')
 
 ax[2].imshow(image, cmap=cm.gray)
+origin = np.array((0, image.shape[1]))
 for _, angle, dist in zip(*hough_line_peaks(h, theta, d)):
-    y0 = (dist - 0 * np.cos(angle)) / np.sin(angle)
-    y1 = (dist - image.shape[1] * np.cos(angle)) / np.sin(angle)
-    ax[2].plot((0, image.shape[1]), (y0, y1), '-r')
-ax[2].set_xlim((0, image.shape[1]))
+    y0, y1 = (dist - origin * np.cos(angle)) / np.sin(angle)
+    ax[2].plot(origin, (y0, y1), '-r')
+ax[2].set_xlim(origin)
 ax[2].set_ylim((image.shape[0], 0))
 ax[2].set_axis_off()
 ax[2].set_title('Detected lines')
@@ -104,6 +111,13 @@ ax[2].set_title('Detected lines')
 plt.tight_layout()
 plt.show()
 
+
+###############################
+# Probabilistic Hough Transform
+# =============================
+
+from skimage.transform import probabilistic_hough_line
+
 # Line finding using the Probabilistic Hough Transform
 image = data.camera()
 edges = canny(image, 2, 1, 25)
