@@ -11,7 +11,6 @@ whether the surrounding points are greater than or less than the central point
 Before trying out LBP on an image, it helps to look at a schematic of LBPs.
 The below code is just used to plot the schematic.
 """
-from __future__ import print_function
 import numpy as np
 import matplotlib.pyplot as plt
 
@@ -102,13 +101,13 @@ def highlight_bars(bars, indexes):
         bars[i].set_facecolor('r')
 
 
-image = data.load('brick.png')
+image = data.brick()
 lbp = local_binary_pattern(image, n_points, radius, METHOD)
 
 
 def hist(ax, lbp):
     n_bins = int(lbp.max() + 1)
-    return ax.hist(lbp.ravel(), normed=True, bins=n_bins, range=(0, n_bins),
+    return ax.hist(lbp.ravel(), density=True, bins=n_bins, range=(0, n_bins),
                    facecolor='0.5')
 
 
@@ -133,8 +132,8 @@ for ax, labels in zip(ax_img, label_sets):
 for ax, labels, name in zip(ax_hist, label_sets, titles):
     counts, _, bars = hist(ax, lbp)
     highlight_bars(bars, labels)
-    ax.set_ylim(ymax=np.max(counts[:-1]))
-    ax.set_xlim(xmax=n_points + 2)
+    ax.set_ylim(top=np.max(counts[:-1]))
+    ax.set_xlim(right=n_points + 2)
     ax.set_title(name)
 
 ax_hist[0].set_ylabel('Percentage')
@@ -167,9 +166,9 @@ def match(refs, img):
     best_name = None
     lbp = local_binary_pattern(img, n_points, radius, METHOD)
     n_bins = int(lbp.max() + 1)
-    hist, _ = np.histogram(lbp, normed=True, bins=n_bins, range=(0, n_bins))
+    hist, _ = np.histogram(lbp, density=True, bins=n_bins, range=(0, n_bins))
     for name, ref in refs.items():
-        ref_hist, _ = np.histogram(ref, normed=True, bins=n_bins,
+        ref_hist, _ = np.histogram(ref, density=True, bins=n_bins,
                                    range=(0, n_bins))
         score = kullback_leibler_divergence(hist, ref_hist)
         if score < best_score:
@@ -178,14 +177,14 @@ def match(refs, img):
     return best_name
 
 
-brick = data.load('brick.png')
-grass = data.load('grass.png')
-wall = data.load('rough-wall.png')
+brick = data.brick()
+grass = data.grass()
+gravel = data.gravel()
 
 refs = {
     'brick': local_binary_pattern(brick, n_points, radius, METHOD),
     'grass': local_binary_pattern(grass, n_points, radius, METHOD),
-    'wall': local_binary_pattern(wall, n_points, radius, METHOD)
+    'gravel': local_binary_pattern(gravel, n_points, radius, METHOD)
 }
 
 # classify rotated textures
@@ -212,8 +211,8 @@ ax2.axis('off')
 hist(ax5, refs['grass'])
 ax5.set_xlabel('Uniform LBP values')
 
-ax3.imshow(wall)
+ax3.imshow(gravel)
 ax3.axis('off')
-hist(ax6, refs['wall'])
+hist(ax6, refs['gravel'])
 
 plt.show()
