@@ -12,14 +12,19 @@ detect subtle variations in the local gray level distribution.
 
 In the first example, the image is composed of two surfaces with two slightly
 different distributions. The image has a uniform random distribution in the
-range [-14, +14] in the middle of the image and a uniform random distribution in
-the range [-15, 15] at the image borders, both centered at a gray value of 128.
+range [-15, +15] in the middle of the image and a uniform random distribution in
+the range [-14, 14] at the image borders, both centered at a gray value of 128.
 To detect the central square, we compute the local entropy measure using a
 circular structuring element of a radius big enough to capture the local gray
 level distribution. The second example shows how to detect texture in the camera
 image using a smaller structuring element.
 
 """
+
+######################################################################
+# Object detection
+# ================
+
 import matplotlib.pyplot as plt
 import numpy as np
 
@@ -28,8 +33,6 @@ from skimage.util import img_as_ubyte
 from skimage.filters.rank import entropy
 from skimage.morphology import disk
 
-# First example: object detection.
-
 noise_mask = np.full((128, 128), 28, dtype=np.uint8)
 noise_mask[32:-32, 32:-32] = 30
 
@@ -41,16 +44,18 @@ entr_img = entropy(img, disk(10))
 
 fig, (ax0, ax1, ax2) = plt.subplots(nrows=1, ncols=3, figsize=(10, 4))
 
-ax0.imshow(noise_mask, cmap='gray')
-ax0.set_xlabel("Noise mask")
+img0 = ax0.imshow(noise_mask, cmap='gray')
+ax0.set_title("Object")
 ax1.imshow(img, cmap='gray')
-ax1.set_xlabel("Noisy image")
+ax1.set_title("Noisy image")
 ax2.imshow(entr_img, cmap='viridis')
-ax2.set_xlabel("Local entropy")
+ax2.set_title("Local entropy")
 
 fig.tight_layout()
 
-# Second example: texture detection.
+######################################################################
+# Texture detection
+# =================
 
 image = img_as_ubyte(data.camera())
 
