@@ -3,8 +3,10 @@
  * Reference [2]: https://doi.org/10.1162/089976600300015033
  * Additional improvements by Leonid Bloch. */
 
+#include <stdint.h>
+
 /* use just EXP_A = 1512775 for integer version, to avoid FP calculations */
-#define EXP_A (1512775.3951951856938)  /* 2^20*ln2 */
+#define EXP_A (1512775.3951951856938)  /* 2^20/ln2 */
 /* For min. RMS error */
 #define EXP_BC 1072632447              /* 1023*2^20 - 60801 */
 /* For min. max. relative error */
@@ -17,7 +19,7 @@ __inline double fast_exp (double y)
     union
     {
         double d;
-        struct { int i, j; } n;
+        struct { int32_t i, j; } n;
         char t[8];
     } _eco;
 
@@ -26,15 +28,20 @@ __inline double fast_exp (double y)
     switch(_eco.t[0]) {
         case 1:
             /* Little endian */
-            _eco.n.j = (int)(EXP_A*(y)) + EXP_BC;
+            _eco.n.j = (int32_t)(EXP_A*(y)) + EXP_BC;
             _eco.n.i = 0;
             break;
         case 0:
             /* Big endian */
-            _eco.n.i = (int)(EXP_A*(y)) + EXP_BC;
+            _eco.n.i = (int32_t)(EXP_A*(y)) + EXP_BC;
             _eco.n.j = 0;
             break;
     }
 
     return _eco.d;
 }
+
+__inline float fast_expf (float y)
+{
+  return (float)fast_exp((double)y);
+}
