@@ -2,7 +2,7 @@ import itertools
 
 import numpy as np
 
-from .._shared.utils import warn
+from .._shared.utils import warn, change_default_value
 from ..util import img_as_float
 from . import rgb_colors
 from .colorconv import rgb2gray, gray2rgb
@@ -45,7 +45,7 @@ def _match_label_with_color(label, colors, bg_label, bg_color):
     # Temporarily set background color; it will be removed later.
     if bg_color is None:
         bg_color = (0, 0, 0)
-    bg_color = _rgb_vector([bg_color])
+    bg_color = _rgb_vector(bg_color)
 
     # map labels to their ranks among all labels from small to large
     unique_labels, mapped_labels = np.unique(label, return_inverse=True)
@@ -66,11 +66,12 @@ def _match_label_with_color(label, colors, bg_label, bg_color):
 
     # Modify labels and color cycle so background color is used only once.
     color_cycle = itertools.cycle(colors)
-    color_cycle = itertools.chain(bg_color, color_cycle)
+    color_cycle = itertools.chain([bg_color], color_cycle)
 
     return mapped_labels, color_cycle
 
 
+@change_default_value("bg_label", new_value=0, changed_version="0.19")
 def label2rgb(label, image=None, colors=None, alpha=0.3,
               bg_label=-1, bg_color=(0, 0, 0), image_alpha=1, kind='overlay'):
     """Return an RGB image where color-coded labels are painted over the image.
@@ -88,7 +89,9 @@ def label2rgb(label, image=None, colors=None, alpha=0.3,
     alpha : float [0, 1], optional
         Opacity of colorized labels. Ignored if image is `None`.
     bg_label : int, optional
-        Label that's treated as the background.
+        Label that's treated as the background. If `bg_label` is specified,
+        `bg_color` is `None`, and `kind` is `overlay`,
+        background is not painted by any colors.
     bg_color : str or array, optional
         Background color. Must be a name in `color_dict` or RGB float values
         between [0, 1].
@@ -132,7 +135,8 @@ def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
     alpha : float [0, 1], optional
         Opacity of colorized labels. Ignored if image is `None`.
     bg_label : int, optional
-        Label that's treated as the background.
+        Label that's treated as the background. If `bg_label` is specified and
+        `bg_color` is `None`, background is not painted by any colors.
     bg_color : str or array, optional
         Background color. Must be a name in `color_dict` or RGB float values
         between [0, 1].
@@ -160,7 +164,10 @@ def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
         if image.min() < 0:
             warn("Negative intensities in `image` are not supported")
 
-        image = img_as_float(rgb2gray(image))
+        if image.ndim > label.ndim:
+            image = img_as_float(rgb2gray(image))
+        else:
+            image = img_as_float(image)
         image = gray2rgb(image) * image_alpha + (1 - image_alpha)
 
     # Ensure that all labels are non-negative so we can index into
@@ -171,7 +178,7 @@ def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
         bg_label -= offset
 
     new_type = np.min_scalar_type(int(label.max()))
-    if new_type == np.bool:
+    if new_type == bool:
         new_type = np.uint8
     label = label.astype(new_type)
 
@@ -183,7 +190,7 @@ def _label2rgb_overlay(label, image=None, colors=None, alpha=0.3,
 
     dense_labels = range(max(mapped_labels_flat) + 1)
 
-    label_to_color = np.array([c for i, c in zip(dense_labels, color_cycle)])
+    label_to_color = np.stack([c for i, c in zip(dense_labels, color_cycle)])
 
     mapped_labels = label
     mapped_labels.flat = mapped_labels_flat
@@ -216,7 +223,7 @@ def _label2rgb_avg(label_field, image, bg_label=0, bg_color=(0, 0, 0)):
     out : array, same shape and type as `image`
         The output visualization.
     """
-    out = np.zeros_like(image)
+    out = np.zeros(label_field.shape + (3,))
     labels = np.unique(label_field)
     bg = (labels == bg_label)
     if bg.any():
