@@ -1,3 +1,4 @@
+import warnings
 import numpy as np
 
 from .._shared._geometry import polygon_clip
@@ -148,28 +149,65 @@ def circle(r, c, radius, shape=None):
     Parameters
     ----------
     r, c : double
-        Centre coordinate of circle.
+        Center coordinate of disk.
     radius : double
-        Radius of circle.
+        Radius of disk.
     shape : tuple, optional
         Image shape which is used to determine the maximum extent of output
-        pixel coordinates. This is useful for circles that exceed the image
-        size. If None, the full extent of the circle is used.  Must be at least
+        pixel coordinates. This is useful for disks that exceed the image
+        size. If None, the full extent of the disk is used.  Must be at least
         length 2. Only the first two values are used to determine the extent of
         the input image.
 
     Returns
     -------
     rr, cc : ndarray of int
-        Pixel coordinates of circle.
+        Pixel coordinates of disk.
+        May be used to directly index into an array, e.g.
+        ``img[rr, cc] = 1``.
+
+    Warns
+    -----
+    Deprecated:
+        .. versionadded:: 0.17
+
+            This function is deprecated and will be removed in scikit-image 0.19.
+            Please use the function named ``disk`` instead.
+    """
+    warnings.warn("`draw.circle` is deprecated in favor of `draw.disk`."
+                  "`draw.circle` will be removed in version 0.19",
+                  FutureWarning, stacklevel=2)
+    return disk((r, c), radius, shape=shape)
+
+
+def disk(center, radius, *, shape=None):
+    """Generate coordinates of pixels within circle.
+
+    Parameters
+    ----------
+    center : tuple
+        Center coordinate of disk.
+    radius : double
+        Radius of disk.
+    shape : tuple, optional
+        Image shape which is used to determine the maximum extent of output
+        pixel coordinates. This is useful for disks that exceed the image
+        size. If None, the full extent of the disk is used.  Must be at least
+        length 2. Only the first two values are used to determine the extent of
+        the input image.
+
+    Returns
+    -------
+    rr, cc : ndarray of int
+        Pixel coordinates of disk.
         May be used to directly index into an array, e.g.
         ``img[rr, cc] = 1``.
 
     Examples
     --------
-    >>> from skimage.draw import circle
+    >>> from skimage.draw import disk
     >>> img = np.zeros((10, 10), dtype=np.uint8)
-    >>> rr, cc = circle(4, 4, 5)
+    >>> rr, cc = disk((4, 4), 5)
     >>> img[rr, cc] = 1
     >>> img
     array([[0, 0, 1, 1, 1, 1, 1, 0, 0, 0],
@@ -183,6 +221,7 @@ def circle(r, c, radius, shape=None):
            [0, 0, 1, 1, 1, 1, 1, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], dtype=uint8)
     """
+    r, c = center
     return ellipse(r, c, radius, radius, shape)
 
 
@@ -446,14 +485,14 @@ def polygon(r, c, shape=None):
     >>> img[rr, cc] = 1
     >>> img
     array([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-           [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-           [0, 0, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 1, 0, 0],
            [0, 0, 1, 1, 1, 1, 1, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 0, 0, 0],
            [0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
-           [0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
+           [0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
            [0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
-           [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], dtype=uint8)
 
     """
@@ -467,7 +506,7 @@ def circle_perimeter(r, c, radius, method='bresenham', shape=None):
     ----------
     r, c : int
         Centre coordinate of circle.
-    radius: int
+    radius : int
         Radius of circle.
     method : {'bresenham', 'andres'}, optional
         bresenham : Bresenham method (default)
@@ -530,7 +569,7 @@ def circle_perimeter_aa(r, c, radius, shape=None):
     ----------
     r, c : int
         Centre coordinate of circle.
-    radius: int
+    radius : int
         Radius of circle.
     shape : tuple, optional
         Image shape which is used to determine the maximum extent of output
@@ -729,7 +768,7 @@ def rectangle(start, end=None, extent=None, shape=None):
         ``([num_planes,] num_rows, num_cols)``.
         Either `end` or `extent` must be specified.
         A negative extent is valid, and will result in a rectangle
-        going along the oposite direction. If extent is negative, the
+        going along the opposite direction. If extent is negative, the
         `start` point is not included.
     shape : tuple, optional
         Image shape used to determine the maximum bounds of the output
