@@ -10,7 +10,7 @@ def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration, get_numpy_include_dirs
 
     config = Configuration('feature', parent_package, top_path)
-    config.add_data_dir('tests')
+    config.add_data_files('orb_descriptor_positions.txt')
 
     cython(['corner_cy.pyx',
             'censure_cy.pyx',
