@@ -1,9 +1,10 @@
+import pytest
+
 import numpy as np
-from skimage.draw import circle
+from skimage.draw import disk
 from skimage.draw.draw3d import ellipsoid
 from skimage.feature import blob_dog, blob_log, blob_doh
 from skimage.feature.blob import _blob_overlap
-from skimage import util
 import math
 from numpy.testing import assert_almost_equal
 
@@ -12,13 +13,13 @@ def test_blob_dog():
     r2 = math.sqrt(2)
     img = np.ones((512, 512))
 
-    xs, ys = circle(400, 130, 5)
+    xs, ys = disk((400, 130), 5)
     img[xs, ys] = 255
 
-    xs, ys = circle(100, 300, 25)
+    xs, ys = disk((100, 300), 25)
     img[xs, ys] = 255
 
-    xs, ys = circle(200, 350, 45)
+    xs, ys = disk((200, 350), 45)
     img[xs, ys] = 255
 
     blobs = blob_dog(img, min_sigma=5, max_sigma=50)
@@ -49,7 +50,7 @@ def test_blob_dog():
     r = 10
     pad = 10
     im3 = ellipsoid(r, r, r)
-    im3 = util.pad(im3, pad, mode='constant')
+    im3 = np.pad(im3, pad, mode='constant')
 
     blobs = blob_dog(im3, min_sigma=3, max_sigma=10,
                           sigma_ratio=1.2, threshold=0.1)
@@ -65,7 +66,7 @@ def test_blob_dog():
     r = 10
     pad = 10
     im3 = ellipsoid(r / 2, r, r)
-    im3 = util.pad(im3, pad, mode='constant')
+    im3 = np.pad(im3, pad, mode='constant')
 
     blobs = blob_dog(
         im3,
@@ -88,13 +89,13 @@ def test_blob_dog():
 
     # image where blob is 5 px from borders, radius 5
     img = np.ones((512, 512))
-    xs, ys = circle(5, 5, 5)
+    xs, ys = disk((5, 5), 5)
     img[xs, ys] = 255
 
 
 def test_blob_dog_excl_border():
     img = np.ones((512, 512))
-    xs, ys = circle(5, 5, 5)
+    xs, ys = disk((5, 5), 5)
     img[xs, ys] = 255
     blobs = blob_dog(
         img,
@@ -111,7 +112,7 @@ def test_blob_dog_excl_border():
         min_sigma=1.5,
         max_sigma=5,
         sigma_ratio=1.2,
-        exclude_border=5
+        exclude_border=6,
     )
     msg = "zero blobs should be detected, as only blob is 5 px from border"
     assert blobs.shape[0] == 0, msg
@@ -121,16 +122,16 @@ def test_blob_log():
     r2 = math.sqrt(2)
     img = np.ones((256, 256))
 
-    xs, ys = circle(200, 65, 5)
+    xs, ys = disk((200, 65), 5)
     img[xs, ys] = 255
 
-    xs, ys = circle(80, 25, 15)
+    xs, ys = disk((80, 25), 15)
     img[xs, ys] = 255
 
-    xs, ys = circle(50, 150, 25)
+    xs, ys = disk((50, 150), 25)
     img[xs, ys] = 255
 
-    xs, ys = circle(100, 175, 30)
+    xs, ys = disk((100, 175), 30)
     img[xs, ys] = 255
 
     blobs = blob_log(img, min_sigma=5, max_sigma=20, threshold=1)
@@ -192,12 +193,26 @@ def test_blob_log():
     assert blob_log(img_empty).size == 0
 
 
+def test_blob_log_no_warnings():
+    img = np.ones((11, 11))
+
+    xs, ys = disk((5, 5), 2)
+    img[xs, ys] = 255
+
+    xs, ys = disk((7, 6), 2)
+    img[xs, ys] = 255
+
+    with pytest.warns(None) as records:
+        blob_log(img, max_sigma=20, num_sigma=10, threshold=.1)
+    assert len(records) == 0
+
+
 def test_blob_log_3d():
     # Testing 3D
     r = 6
     pad = 10
     im3 = ellipsoid(r, r, r)
-    im3 = util.pad(im3, pad, mode='constant')
+    im3 = np.pad(im3, pad, mode='constant')
 
     blobs = blob_log(im3, min_sigma=3, max_sigma=10)
     b = blobs[0]
@@ -214,7 +229,7 @@ def test_blob_log_3d_anisotropic():
     r = 6
     pad = 10
     im3 = ellipsoid(r / 2, r, r)
-    im3 = util.pad(im3, pad, mode='constant')
+    im3 = np.pad(im3, pad, mode='constant')
 
     blobs = blob_log(
         im3,
@@ -235,7 +250,7 @@ def test_blob_log_3d_anisotropic():
 def test_blob_log_exclude_border():
     # image where blob is 5 px from borders, radius 5
     img = np.ones((512, 512))
-    xs, ys = circle(5, 5, 5)
+    xs, ys = disk((5, 5), 5)
     img[xs, ys] = 255
 
     blobs = blob_log(
@@ -247,11 +262,11 @@ def test_blob_log_exclude_border():
     b = blobs[0]
     assert b[0] == b[1] == 5, "blob should be 5 px from x and y borders"
 
-    blobs = blob_dog(
+    blobs = blob_log(
         img,
         min_sigma=1.5,
         max_sigma=5,
-        exclude_border=5
+        exclude_border=6,
     )
     msg = "zero blobs should be detected, as only blob is 5 px from border"
     assert blobs.shape[0] == 0, msg
@@ -260,16 +275,16 @@ def test_blob_log_exclude_border():
 def test_blob_doh():
     img = np.ones((512, 512), dtype=np.uint8)
 
-    xs, ys = circle(400, 130, 20)
+    xs, ys = disk((400, 130), 20)
     img[xs, ys] = 255
 
-    xs, ys = circle(460, 50, 30)
+    xs, ys = disk((460, 50), 30)
     img[xs, ys] = 255
 
-    xs, ys = circle(100, 300, 40)
+    xs, ys = disk((100, 300), 40)
     img[xs, ys] = 255
 
-    xs, ys = circle(200, 350, 50)
+    xs, ys = disk((200, 350), 50)
     img[xs, ys] = 255
 
     blobs = blob_doh(
@@ -307,16 +322,16 @@ def test_blob_doh():
 def test_blob_doh_log_scale():
     img = np.ones((512, 512), dtype=np.uint8)
 
-    xs, ys = circle(400, 130, 20)
+    xs, ys = disk((400, 130), 20)
     img[xs, ys] = 255
 
-    xs, ys = circle(460, 50, 30)
+    xs, ys = disk((460, 50), 30)
     img[xs, ys] = 255
 
-    xs, ys = circle(100, 300, 40)
+    xs, ys = disk((100, 300), 40)
     img[xs, ys] = 255
 
-    xs, ys = circle(200, 350, 50)
+    xs, ys = disk((200, 350), 50)
     img[xs, ys] = 255
 
     blobs = blob_doh(
@@ -336,12 +351,12 @@ def test_blob_doh_log_scale():
     assert abs(b[1] - 130) <= thresh
     assert abs(radius(b) - 20) <= thresh
 
-    b = s[1]
+    b = s[2]
     assert abs(b[0] - 460) <= thresh
     assert abs(b[1] - 50) <= thresh
     assert abs(radius(b) - 30) <= thresh
 
-    b = s[2]
+    b = s[1]
     assert abs(b[0] - 100) <= thresh
     assert abs(b[1] - 300) <= thresh
     assert abs(radius(b) - 40) <= thresh
@@ -361,10 +376,10 @@ def test_blob_doh_no_peaks():
 def test_blob_doh_overlap():
     img = np.ones((256, 256), dtype=np.uint8)
 
-    xs, ys = circle(100, 100, 20)
+    xs, ys = disk((100, 100), 20)
     img[xs, ys] = 255
 
-    xs, ys = circle(120, 100, 30)
+    xs, ys = disk((120, 100), 30)
     img[xs, ys] = 255
 
     blobs = blob_doh(
@@ -382,11 +397,11 @@ def test_blob_log_overlap_3d():
     r1, r2 = 7, 6
     pad1, pad2 = 11, 12
     blob1 = ellipsoid(r1, r1, r1)
-    blob1 = util.pad(blob1, pad1, mode='constant')
+    blob1 = np.pad(blob1, pad1, mode='constant')
     blob2 = ellipsoid(r2, r2, r2)
-    blob2 = util.pad(blob2, [(pad2, pad2), (pad2 - 9, pad2 + 9),
-                                           (pad2, pad2)],
-                            mode='constant')
+    blob2 = np.pad(blob2, [(pad2, pad2), (pad2 - 9, pad2 + 9),
+                           (pad2, pad2)],
+                   mode='constant')
     im3 = np.logical_or(blob1, blob2)
 
     blobs = blob_log(im3,  min_sigma=2, max_sigma=10, overlap=0.1)
@@ -422,11 +437,11 @@ def test_blob_log_overlap_3d_anisotropic():
     r1, r2 = 7, 6
     pad1, pad2 = 11, 12
     blob1 = ellipsoid(r1, r1, r1)
-    blob1 = util.pad(blob1, pad1, mode='constant')
+    blob1 = np.pad(blob1, pad1, mode='constant')
     blob2 = ellipsoid(r2, r2, r2)
-    blob2 = util.pad(blob2, [(pad2, pad2), (pad2 - 9, pad2 + 9),
-                             (pad2, pad2)],
-                     mode='constant')
+    blob2 = np.pad(blob2, [(pad2, pad2), (pad2 - 9, pad2 + 9),
+                           (pad2, pad2)],
+                   mode='constant')
     im3 = np.logical_or(blob1, blob2)
 
     blobs = blob_log(im3, min_sigma=[2, 2.01, 2.005],
@@ -439,6 +454,7 @@ def test_blob_log_overlap_3d_anisotropic():
     assert_almost_equal(overlap,
                         1./math.pi * (2 * math.acos(1./2) - math.sqrt(3)/2.))
 
+
 def test_no_blob():
     im = np.zeros((10, 10))
     blobs = blob_log(im,  min_sigma=2, max_sigma=5, num_sigma=4)
