@@ -1,18 +1,32 @@
-import os
 import numpy as np
 from skimage._shared.testing import (assert_equal, assert_array_equal,
-                                     assert_allclose)
+                                     assert_allclose,
+                                     assert_array_almost_equal)
 from skimage._shared import testing
 
-import skimage
 from skimage.util import img_as_ubyte, img_as_float
 from skimage import data, util, morphology
-from skimage.morphology import grey, disk
+from skimage.morphology import grey, disk, ball
 from skimage.filters import rank
 from skimage.filters.rank import __all__ as all_rank_filters
+from skimage.filters.rank import subtract_mean
 from skimage._shared._warnings import expected_warnings
-from skimage._shared.testing import test_parallel, arch32, parametrize, xfail
-from pytest import param
+from skimage._shared.testing import test_parallel, parametrize, fetch
+import pytest
+
+# To be removed along with tophat and bottomhat functions.
+all_rank_filters.remove('tophat')
+all_rank_filters.remove('bottomhat')
+
+
+def test_deprecation():
+    selem = disk(3)
+    image = img_as_ubyte(data.camera()[:50, :50])
+
+    with expected_warnings(['rank.tophat is deprecated.']):
+        rank.tophat(image, selem)
+    with expected_warnings(['rank.bottomhat is deprecated.']):
+        rank.bottomhat(image, selem)
 
 
 def test_otsu_edge_case():
@@ -41,6 +55,26 @@ def test_otsu_edge_case():
     assert result[1, 1] in [141, 172]
 
 
+@pytest.mark.parametrize("dtype", [np.uint8, np.uint16])
+def test_subtract_mean_underflow_correction(dtype):
+    # Input: [10, 10, 10]
+    selem = np.ones((1, 3))
+    arr = np.array([[10, 10, 10]], dtype=dtype)
+    result = subtract_mean(arr, selem)
+
+    if dtype == np.uint8:
+        expected_val = 127
+    else:
+        expected_val = (arr.max() + 1) // 2 - 1
+
+    assert np.all(result == expected_val)
+
+
+# Note: Explicitly read all values into a dict. Otherwise, stochastic test
+#       failures related to I/O can occur during parallel test cases.
+ref_data = dict(np.load(fetch("data/rank_filter_tests.npz")))
+ref_data_3d = dict(np.load(fetch('data/rank_filters_tests_3d.npz')))
+
 
 class TestRank():
     def setup(self):
@@ -48,11 +82,14 @@ class TestRank():
         # This image is used along with @test_parallel
         # to ensure that the same seed is used for each thread.
         self.image = np.random.rand(25, 25)
+        np.random.seed(0)
+        self.volume = np.random.rand(10, 10, 10)
         # Set again the seed for the other tests.
         np.random.seed(0)
         self.selem = morphology.disk(1)
-        self.refs = np.load(os.path.join(skimage.data_dir,
-                                         "rank_filter_tests.npz"))
+        self.selem_3d = morphology.ball(1)
+        self.refs = ref_data
+        self.refs_3d = ref_data_3d
 
     @parametrize('filter', all_rank_filters)
     def test_rank_filter(self, filter):
@@ -79,9 +116,23 @@ class TestRank():
                 # reason.
                 assert result[19, 18] in [141, 172]
                 result[19, 18] = 172
-                assert_array_equal(expected, result)
+                assert_array_almost_equal(expected, result)
             else:
-                assert_array_equal(expected, result)
+                assert_array_almost_equal(expected, result)
+
+        check()
+
+    @parametrize('filter', ['equalize', 'otsu', 'autolevel', 'gradient',
+                            'majority', 'maximum', 'mean', 'geometric_mean',
+                            'subtract_mean', 'median', 'minimum', 'modal',
+                            'enhance_contrast', 'pop', 'sum', 'threshold',
+                            'noise_filter', 'entropy'])
+    def test_rank_filters_3D(self, filter):
+        @test_parallel(warnings_matching=['Possible precision loss'])
+        def check():
+            expected = self.refs_3d[filter]
+            result = getattr(rank, filter)(self.volume, self.selem_3d)
+            assert_array_almost_equal(expected, result)
 
         check()
 
@@ -132,7 +183,6 @@ class TestRank():
                                  selem=elem, shift_x=+1, shift_y=+1, p0=.1, p1=.9)
             assert_equal(image16.shape, out16.shape)
 
-
     def test_compare_with_grey_dilation(self):
         # compare the result of maximum filter with dilate
 
@@ -146,7 +196,6 @@ class TestRank():
             cm = grey.dilation(image=image, selem=elem)
             assert_equal(out, cm)
 
-
     def test_compare_with_grey_erosion(self):
         # compare the result of maximum filter with erode
 
@@ -160,7 +209,6 @@ class TestRank():
             cm = grey.erosion(image=image, selem=elem)
             assert_equal(out, cm)
 
-
     def test_bitdepth(self):
         # test the different bit depth for rank16
 
@@ -179,7 +227,6 @@ class TestRank():
                 rank.mean_percentile(image=image, selem=elem, mask=mask,
                                      out=out, shift_x=0, shift_y=0, p0=.1, p1=.9)
 
-
     def test_population(self):
         # check the number of valid pixels in the neighborhood
 
@@ -196,7 +243,6 @@ class TestRank():
                       [4, 6, 6, 6, 4]])
         assert_equal(r, out)
 
-
     def test_structuring_element8(self):
         # check the output for a custom structuring element
 
@@ -227,7 +273,6 @@ class TestRank():
                      shift_x=1, shift_y=1)
         assert_equal(r, out)
 
-
     def test_pass_on_bitdepth(self):
         # should pass because data bitdepth is not too high for the function
 
@@ -238,7 +283,6 @@ class TestRank():
         with expected_warnings(["Bad rank filter performance"]):
             rank.maximum(image=image, selem=elem, out=out, mask=mask)
 
-
     def test_inplace_output(self):
         # rank filters are not supposed to filter inplace
 
@@ -248,7 +292,6 @@ class TestRank():
         with testing.raises(NotImplementedError):
             rank.mean(image, selem, out=out)
 
-
     def test_compare_autolevels(self):
         # compare autolevel and percentile autolevel with p0=0.0 and p1=1.0
         # should returns the same arrays
@@ -262,10 +305,9 @@ class TestRank():
 
         assert_equal(loc_autolevel, loc_perc_autolevel)
 
-
     def test_compare_autolevels_16bit(self):
-        # compare autolevel(16-bit) and percentile autolevel(16-bit) with p0=0.0
-        # and p1=1.0 should returns the same arrays
+        # compare autolevel(16-bit) and percentile autolevel(16-bit) with
+        # p0=0.0 and p1=1.0 should returns the same arrays
 
         image = data.camera().astype(np.uint16) * 4
 
@@ -276,15 +318,14 @@ class TestRank():
 
         assert_equal(loc_autolevel, loc_perc_autolevel)
 
-
     def test_compare_ubyte_vs_float(self):
 
         # Create signed int8 image that and convert it to uint8
         image_uint = img_as_ubyte(data.camera()[:50, :50])
         image_float = img_as_float(image_uint)
 
-        methods = ['autolevel', 'bottomhat', 'equalize', 'gradient', 'threshold',
-                   'subtract_mean', 'enhance_contrast', 'pop', 'tophat']
+        methods = ['autolevel', 'equalize', 'gradient', 'threshold',
+                   'subtract_mean', 'enhance_contrast', 'pop']
 
         for method in methods:
             func = getattr(rank, method)
@@ -293,6 +334,26 @@ class TestRank():
                 out_f = func(image_float, disk(3))
             assert_equal(out_u, out_f)
 
+    def test_compare_ubyte_vs_float_3d(self):
+
+        # Create signed int8 volume that and convert it to uint8
+        np.random.seed(0)
+        volume_uint = np.random.randint(0, high=256,
+                                        size=(10, 20, 30), dtype=np.uint8)
+        volume_float = img_as_float(volume_uint)
+
+        methods_3d = ['equalize', 'otsu', 'autolevel', 'gradient',
+                     'majority', 'maximum', 'mean', 'geometric_mean',
+                     'subtract_mean', 'median', 'minimum', 'modal',
+                     'enhance_contrast', 'pop', 'sum', 'threshold',
+                     'noise_filter', 'entropy']
+
+        for method in methods_3d:
+            func = getattr(rank, method)
+            out_u = func(volume_uint, ball(3))
+            with expected_warnings(["Possible precision loss"]):
+                out_f = func(volume_float, ball(3))
+            assert_equal(out_u, out_f)
 
     def test_compare_8bit_unsigned_vs_signed(self):
         # filters applied on 8-bit image ore 16-bit image (having only real 8-bit
@@ -305,9 +366,9 @@ class TestRank():
         image_u = img_as_ubyte(image_s)
         assert_equal(image_u, img_as_ubyte(image_s))
 
-        methods = ['autolevel', 'bottomhat', 'equalize', 'gradient', 'maximum',
+        methods = ['autolevel', 'equalize', 'gradient', 'maximum',
                    'mean', 'geometric_mean', 'subtract_mean', 'median', 'minimum',
-                   'modal', 'enhance_contrast', 'pop', 'threshold', 'tophat']
+                   'modal', 'enhance_contrast', 'pop', 'threshold']
 
         for method in methods:
             func = getattr(rank, method)
@@ -316,10 +377,34 @@ class TestRank():
                 out_s = func(image_s, disk(3))
             assert_equal(out_u, out_s)
 
+    def test_compare_8bit_unsigned_vs_signed_3d(self):
+        # filters applied on 8-bit volume ore 16-bit volume (having only real 8-bit
+        # of dynamic) should be identical
+
+        # Create signed int8 volume that and convert it to uint8
+        np.random.seed(0)
+        volume_s = np.random.randint(0, high=127,
+                                     size=(10, 20, 30), dtype=np.int8)
+        volume_u = img_as_ubyte(volume_s)
+        assert_equal(volume_u, img_as_ubyte(volume_s))
+
+        methods_3d = ['equalize', 'otsu', 'autolevel', 'gradient',
+                     'majority', 'maximum', 'mean', 'geometric_mean',
+                     'subtract_mean', 'median', 'minimum', 'modal',
+                     'enhance_contrast', 'pop', 'sum', 'threshold',
+                     'noise_filter', 'entropy']
+
+        for method in methods_3d:
+            func = getattr(rank, method)
+            out_u = func(volume_u, ball(3))
+            with expected_warnings(["Possible precision loss"]):
+                out_s = func(volume_s, ball(3))
+            assert_equal(out_u, out_s)
+
     @parametrize('method',
-                 ['autolevel', 'bottomhat', 'equalize', 'gradient', 'maximum',
+                 ['autolevel', 'equalize', 'gradient', 'maximum',
                   'mean', 'subtract_mean', 'median', 'minimum', 'modal',
-                  'enhance_contrast', 'pop', 'threshold', 'tophat'])
+                  'enhance_contrast', 'pop', 'threshold'])
     def test_compare_8bit_vs_16bit(self, method):
         # filters applied on 8-bit image ore 16-bit image (having only real 8-bit
         # of dynamic) should be identical
@@ -327,11 +412,27 @@ class TestRank():
         image16 = image8.astype(np.uint16)
         assert_equal(image8, image16)
 
+        np.random.seed(0)
+        volume8 = np.random.randint(128, high=256,
+                                    size=(10, 10, 10), dtype=np.uint8)
+        volume16 = volume8.astype(np.uint16)
+
+        methods_3d = ['equalize', 'otsu', 'autolevel', 'gradient',
+                     'majority', 'maximum', 'mean', 'geometric_mean',
+                     'subtract_mean', 'median', 'minimum', 'modal',
+                     'enhance_contrast', 'pop', 'sum', 'threshold',
+                     'noise_filter', 'entropy']
+
         func = getattr(rank, method)
         f8 = func(image8, disk(3))
         f16 = func(image16, disk(3))
         assert_equal(f8, f16)
 
+        if (method in methods_3d):
+            f8 = func(volume8, ball(3))
+            f16 = func(volume16, ball(3))
+
+            assert_equal(f8, f16)
 
     def test_trivial_selem8(self):
         # check that min, max and mean returns identity if structuring element
@@ -358,7 +459,6 @@ class TestRank():
                      shift_x=0, shift_y=0)
         assert_equal(image, out)
 
-
     def test_trivial_selem16(self):
         # check that min, max and mean returns identity if structuring element
         # contains only central pixel
@@ -384,7 +484,6 @@ class TestRank():
                      shift_x=0, shift_y=0)
         assert_equal(image, out)
 
-
     def test_smallest_selem8(self):
         # check that min, max and mean returns identity if structuring element
         # contains only central pixel
@@ -407,7 +506,6 @@ class TestRank():
                      shift_x=0, shift_y=0)
         assert_equal(image, out)
 
-
     def test_smallest_selem16(self):
         # check that min, max and mean returns identity if structuring element
         # contains only central pixel
@@ -433,7 +531,6 @@ class TestRank():
                      shift_x=0, shift_y=0)
         assert_equal(image, out)
 
-
     def test_empty_selem(self):
         # check that min, max and mean returns zeros if structuring element is
         # empty
@@ -461,7 +558,6 @@ class TestRank():
                      shift_x=0, shift_y=0)
         assert_equal(res, out)
 
-
     def test_otsu(self):
         # test the local Otsu segmentation on a synthetic image
         # (left to right ramp * sinus)
@@ -475,7 +571,6 @@ class TestRank():
         th = 1 * (test >= rank.otsu(test, selem))
         assert_equal(th, res)
 
-
     def test_entropy(self):
         #  verify that entropy is coherent with bitdepth of the input data
 
@@ -520,7 +615,6 @@ class TestRank():
             out = rank.entropy(data, np.ones((16, 16), dtype=np.uint8))
         assert out.dtype == np.double
 
-
     def test_selem_dtypes(self):
 
         image = np.zeros((5, 5), dtype=np.uint8)
@@ -530,7 +624,7 @@ class TestRank():
         image[2, 3] = 128
         image[1, 2] = 16
 
-        for dtype in (np.uint8, np.uint16, np.int32, np.int64,
+        for dtype in (bool, np.uint8, np.uint16, np.int32, np.int64,
                       np.float32, np.float64):
             elem = np.array([[0, 0, 0], [0, 1, 0], [0, 0, 0]], dtype=dtype)
             rank.mean(image=image, selem=elem, out=out, mask=mask,
@@ -543,7 +637,6 @@ class TestRank():
                                  shift_x=0, shift_y=0)
             assert_equal(image, out)
 
-
     def test_16bit(self):
         image = np.zeros((21, 21), dtype=np.uint16)
         selem = np.ones((3, 3), dtype=np.uint8)
@@ -560,7 +653,6 @@ class TestRank():
                 assert rank.maximum(image, selem)[10, 10] == value
                 assert rank.mean(image, selem)[10, 10] == int(value / selem.size)
 
-
     def test_bilateral(self):
         image = np.zeros((21, 21), dtype=np.uint16)
         selem = np.ones((3, 3), dtype=np.uint8)
@@ -574,7 +666,6 @@ class TestRank():
         assert rank.mean_bilateral(image, selem, s0=11, s1=11)[10, 10] == 1005
         assert rank.pop_bilateral(image, selem, s0=11, s1=11)[10, 10] == 2
 
-
     def test_percentile_min(self):
         # check that percentile p0 = 0 is identical to local min
         img = data.camera()
@@ -589,7 +680,6 @@ class TestRank():
         img_min = rank.minimum(img16, selem=selem)
         assert_equal(img_p0, img_min)
 
-
     def test_percentile_max(self):
         # check that percentile p0 = 1 is identical to local max
         img = data.camera()
@@ -604,7 +694,6 @@ class TestRank():
         img_max = rank.maximum(img16, selem=selem)
         assert_equal(img_p0, img_max)
 
-
     def test_percentile_median(self):
         # check that percentile p0 = 0.5 is identical to local median
         img = data.camera()
@@ -619,7 +708,6 @@ class TestRank():
         img_max = rank.median(img16, selem=selem)
         assert_equal(img_p0, img_max)
 
-
     def test_sum(self):
         # check the number of valid pixels in the neighborhood
 
@@ -666,7 +754,6 @@ class TestRank():
             image=image16, selem=elem, out=out16, mask=mask, s0=1000, s1=1000)
         assert_equal(r, out16)
 
-
     def test_windowed_histogram(self):
         # check the number of valid pixels in the neighborhood
 
@@ -706,7 +793,6 @@ class TestRank():
                                                 mask=mask, n_bins=5)
         assert larger_output.shape[2] == 5
 
-
     def test_median_default_value(self):
         a = np.zeros((3, 3), dtype=np.uint8)
         a[1] = 1
@@ -715,10 +801,23 @@ class TestRank():
         assert rank.median(a)[1, 1] == 0
         assert rank.median(a, disk(1))[1, 1] == 1
 
-
     def test_majority(self):
         img = data.camera()
         elem = np.ones((3, 3), dtype=np.uint8)
         expected = rank.windowed_histogram(
             img, elem).argmax(-1).astype(np.uint8)
         assert_equal(expected, rank.majority(img, elem))
+
+    def test_output_same_dtype(self):
+        image = (np.random.rand(100, 100) * 256).astype(np.uint8)
+        out = np.empty_like(image)
+        mask = np.ones(image.shape, dtype=np.uint8)
+        elem = np.ones((3, 3), dtype=np.uint8)
+        rank.maximum(image=image, selem=elem, out=out, mask=mask)
+        assert_equal(image.dtype, out.dtype)
+
+    def test_input_boolean_dtype(self):
+        image = (np.random.rand(100, 100) * 256).astype(bool)
+        elem = np.ones((3, 3), dtype=bool)
+        with testing.raises(ValueError):
+            rank.maximum(image=image, selem=elem)
