@@ -1,13 +1,14 @@
-from __future__ import division
-
+import pytest
 import numpy as np
 from scipy import ndimage as ndi
 
-import skimage
+from skimage import util
 from skimage import data
+from skimage.color import rgb2gray
+from skimage.draw import disk
 from skimage._shared._warnings import expected_warnings
+from skimage.exposure import histogram
 from skimage.filters.thresholding import (threshold_local,
-                                          threshold_adaptive,
                                           threshold_otsu,
                                           threshold_li,
                                           threshold_yen,
@@ -17,11 +18,15 @@ from skimage.filters.thresholding import (threshold_local,
                                           threshold_mean,
                                           threshold_triangle,
                                           threshold_minimum,
+                                          threshold_multiotsu,
                                           try_all_threshold,
                                           _mean_std,
                                           _cross_entropy)
+from skimage.filters._multiotsu import (_get_multiotsu_thresh_indices_lut,
+                                        _get_multiotsu_thresh_indices)
 from skimage._shared import testing
 from skimage._shared.testing import assert_equal, assert_almost_equal
+from skimage._shared.testing import assert_array_equal
 
 
 class TestSimpleImage():
@@ -32,6 +37,16 @@ class TestSimpleImage():
                                [2, 4, 5, 2, 1],
                                [4, 5, 1, 0, 0]], dtype=int)
 
+    def test_minimum(self):
+        with pytest.raises(RuntimeError):
+            threshold_minimum(self.image)
+
+    def test_try_all_threshold(self):
+        fig, ax = try_all_threshold(self.image)
+        all_texts = [axis.texts for axis in ax if axis.texts != []]
+        text_content = [text.get_text() for x in all_texts for text in x]
+        assert 'RuntimeError' in text_content
+
     def test_otsu(self):
         assert threshold_otsu(self.image) == 2
 
@@ -108,32 +123,6 @@ class TestSimpleImage():
         assert all(0.49 < threshold_isodata(imfloat, nbins=1024,
                                             return_all=True))
 
-    def test_threshold_local_equals_adaptive(self):
-        def func(arr):
-            return arr.sum() / arr.shape[0]
-        with expected_warnings(['deprecated', 'return value']):
-            thresholded_original = threshold_adaptive(self.image, 3,
-                                                      method='generic',
-                                                      param=func)
-        threshold_new = threshold_local(self.image, 3, method='generic',
-                                        param=func)
-        assert_equal(thresholded_original, self.image > threshold_new)
-
-    def test_threshold_adaptive_generic(self):
-        def func(arr):
-            return arr.sum() / arr.shape[0]
-        ref = np.array(
-            [[False, False, False, False,  True],
-             [False, False,  True, False,  True],
-             [False, False,  True,  True, False],
-             [False,  True,  True, False, False],
-             [ True,  True, False, False, False]]
-        )
-        with expected_warnings(['deprecated', 'return value']):
-            out = threshold_adaptive(self.image, 3, method='generic',
-                                     param=func)
-        assert_equal(ref, out)
-
     def test_threshold_local_gaussian(self):
         ref = np.array(
             [[False, False, False, False,  True],
@@ -205,45 +194,82 @@ class TestSimpleImage():
         out = self.image > thres
         assert_equal(ref, out)
 
+    def test_threshold_niblack_iterable_window_size(self):
+        ref = np.array(
+            [[False, False, False, True, True],
+             [False, False, True, True, True],
+             [False, True, True, True, False],
+             [False, True, True, True, False],
+             [True, True, False, False, False]]
+        )
+        thres = threshold_niblack(self.image, window_size=[3, 5], k=0.5)
+        out = self.image > thres
+        assert_array_equal(ref, out)
+
+    def test_threshold_sauvola_iterable_window_size(self):
+        ref = np.array(
+            [[False, False, False, True, True],
+             [False, False, True, True, True],
+             [False, False, True, True, False],
+             [False, True, True, True, False],
+             [True, True, False, False, False]]
+        )
+        thres = threshold_sauvola(self.image, window_size=(3, 5), k=0.2, r=128)
+        out = self.image > thres
+        assert_array_equal(ref, out)
+
 
 def test_otsu_camera_image():
-    camera = skimage.img_as_ubyte(data.camera())
-    assert 86 < threshold_otsu(camera) < 88
+    camera = util.img_as_ubyte(data.camera())
+    assert 101 < threshold_otsu(camera) < 103
 
+def test_otsu_camera_image_histogram():
+    camera = util.img_as_ubyte(data.camera())
+    hist = histogram(camera.ravel(), 256, source_range='image')
+    assert 101 < threshold_otsu(hist=hist) < 103
+
+def test_otsu_camera_image_counts():
+    camera = util.img_as_ubyte(data.camera())
+    counts, bin_centers = histogram(camera.ravel(), 256, source_range='image')
+    assert 101 < threshold_otsu(hist=counts) < 103
 
 def test_otsu_coins_image():
-    coins = skimage.img_as_ubyte(data.coins())
+    coins = util.img_as_ubyte(data.coins())
     assert 106 < threshold_otsu(coins) < 108
 
 
 def test_otsu_coins_image_as_float():
-    coins = skimage.img_as_float(data.coins())
+    coins = util.img_as_float(data.coins())
     assert 0.41 < threshold_otsu(coins) < 0.42
 
 
 def test_otsu_astro_image():
-    img = skimage.img_as_ubyte(data.astronaut())
+    img = util.img_as_ubyte(data.astronaut())
     with expected_warnings(['grayscale']):
         assert 109 < threshold_otsu(img) < 111
 
 
 def test_otsu_one_color_image():
     img = np.ones((10, 10), dtype=np.uint8)
-    with testing.raises(ValueError):
-        threshold_otsu(img)
+    assert threshold_otsu(img) == 1
+
+
+def test_otsu_one_color_image_3d():
+    img = np.ones((10, 10, 10), dtype=np.uint8)
+    assert threshold_otsu(img) == 1
 
 
 def test_li_camera_image():
-    image = skimage.img_as_ubyte(data.camera())
+    image = util.img_as_ubyte(data.camera())
     threshold = threshold_li(image)
     ce_actual = _cross_entropy(image, threshold)
-    assert 62 < threshold_li(image) < 63
+    assert 78 < threshold_li(image) < 79
     assert ce_actual < _cross_entropy(image, threshold + 1)
     assert ce_actual < _cross_entropy(image, threshold - 1)
 
 
 def test_li_coins_image():
-    image = skimage.img_as_ubyte(data.coins())
+    image = util.img_as_ubyte(data.coins())
     threshold = threshold_li(image)
     ce_actual = _cross_entropy(image, threshold)
     assert 94 < threshold_li(image) < 95
@@ -252,17 +278,17 @@ def test_li_coins_image():
     # threshold below that found by the iterative method. Not sure why that is
     # but `threshold_li` does find the stationary point of the function (ie the
     # tolerance can be reduced arbitrarily but the exact same threshold is
-    # found), so my guess some kind of histogram binning effect.
+    # found), so my guess is some kind of histogram binning effect.
     assert ce_actual < _cross_entropy(image, threshold - 2)
 
 
 def test_li_coins_image_as_float():
-    coins = skimage.img_as_float(data.coins())
+    coins = util.img_as_float(data.coins())
     assert 94/255 < threshold_li(coins) < 95/255
 
 
 def test_li_astro_image():
-    image = skimage.img_as_ubyte(data.astronaut())
+    image = util.img_as_ubyte(data.astronaut())
     threshold = threshold_li(image)
     ce_actual = _cross_entropy(image, threshold)
     assert 64 < threshold < 65
@@ -290,40 +316,94 @@ def test_li_constant_image_with_nan():
     assert threshold_li(image) == 8
 
 
+def test_li_arbitrary_start_point():
+    cell = data.cell()
+    max_stationary_point = threshold_li(cell)
+    low_stationary_point = threshold_li(cell,
+                                        initial_guess=np.percentile(cell, 5))
+    optimum = threshold_li(cell, initial_guess=np.percentile(cell, 95))
+    assert 67 < max_stationary_point < 68
+    assert 48 < low_stationary_point < 49
+    assert 111 < optimum < 112
+
+
+def test_li_negative_inital_guess():
+    coins = data.coins()
+    with testing.raises(ValueError):
+        result = threshold_li(coins, initial_guess=-5)
+
+
+def test_li_pathological_arrays():
+    # See https://github.com/scikit-image/scikit-image/issues/4140
+    a = np.array([0, 0, 1, 0, 0, 1, 0, 1])
+    b = np.array([0, 0, 0.1, 0, 0, 0.1, 0, 0.1])
+    c = np.array([0, 0, 0.1, 0, 0, 0.1, 0.01, 0.1])
+    d = np.array([0, 0, 1, 0, 0, 1, 0.5, 1])
+    e = np.array([1, 1])
+    f = np.array([1, 2])
+    arrays = [a, b, c, d, e, f]
+    with np.errstate(divide='ignore'):
+        # ignoring "divide by zero encountered in log" error from np.log(0)
+        thresholds = [threshold_li(arr) for arr in arrays]
+    assert np.all(np.isfinite(thresholds))
+
+
 def test_yen_camera_image():
-    camera = skimage.img_as_ubyte(data.camera())
-    assert 197 < threshold_yen(camera) < 199
+    camera = util.img_as_ubyte(data.camera())
+    assert 145 < threshold_yen(camera) < 147
+
+def test_yen_camera_image_histogram():
+    camera = util.img_as_ubyte(data.camera())
+    hist = histogram(camera.ravel(), 256, source_range='image')
+    assert 145 < threshold_yen(hist=hist) < 147
+
+def test_yen_camera_image_counts():
+    camera = util.img_as_ubyte(data.camera())
+    counts, bin_centers = histogram(camera.ravel(), 256, source_range='image')
+    assert 145 < threshold_yen(hist=counts) < 147
 
 
 def test_yen_coins_image():
-    coins = skimage.img_as_ubyte(data.coins())
+    coins = util.img_as_ubyte(data.coins())
     assert 109 < threshold_yen(coins) < 111
 
 
 def test_yen_coins_image_as_float():
-    coins = skimage.img_as_float(data.coins())
+    coins = util.img_as_float(data.coins())
     assert 0.43 < threshold_yen(coins) < 0.44
 
 
-def test_adaptive_even_block_size_error():
+def test_local_even_block_size_error():
     img = data.camera()
     with testing.raises(ValueError):
         threshold_local(img, block_size=4)
 
 
 def test_isodata_camera_image():
-    camera = skimage.img_as_ubyte(data.camera())
+    camera = util.img_as_ubyte(data.camera())
 
     threshold = threshold_isodata(camera)
     assert np.floor((camera[camera <= threshold].mean() +
                      camera[camera > threshold].mean()) / 2.0) == threshold
-    assert threshold == 87
+    assert threshold == 102
 
-    assert threshold_isodata(camera, return_all=True) == [87]
+    assert (threshold_isodata(camera, return_all=True) == [102, 103]).all()
+
+def test_isodata_camera_image_histogram():
+    camera = util.img_as_ubyte(data.camera())
+    hist = histogram(camera.ravel(), 256, source_range='image')
+    threshold = threshold_isodata(hist=hist)
+    assert threshold == 102
+
+def test_isodata_camera_image_counts():
+    camera = util.img_as_ubyte(data.camera())
+    counts, bin_centers = histogram(camera.ravel(), 256, source_range='image')
+    threshold = threshold_isodata(hist=counts)
+    assert threshold == 102
 
 
 def test_isodata_coins_image():
-    coins = skimage.img_as_ubyte(data.coins())
+    coins = util.img_as_ubyte(data.coins())
 
     threshold = threshold_isodata(coins)
     assert np.floor((coins[coins <= threshold].mean() +
@@ -334,7 +414,7 @@ def test_isodata_coins_image():
 
 
 def test_isodata_moon_image():
-    moon = skimage.img_as_ubyte(data.moon())
+    moon = util.img_as_ubyte(data.moon())
 
     threshold = threshold_isodata(moon)
     assert np.floor((moon[moon <= threshold].mean() +
@@ -349,7 +429,7 @@ def test_isodata_moon_image():
 
 
 def test_isodata_moon_image_negative_int():
-    moon = skimage.img_as_ubyte(data.moon()).astype(np.int32)
+    moon = util.img_as_ubyte(data.moon()).astype(np.int32)
     moon -= 100
 
     threshold = threshold_isodata(moon)
@@ -365,7 +445,7 @@ def test_isodata_moon_image_negative_int():
 
 
 def test_isodata_moon_image_negative_float():
-    moon = skimage.img_as_ubyte(data.moon()).astype(np.float64)
+    moon = util.img_as_ubyte(data.moon()).astype(np.float64)
     moon -= 100
 
     assert -14 < threshold_isodata(moon) < -13
@@ -377,15 +457,26 @@ def test_isodata_moon_image_negative_float():
 
 
 def test_threshold_minimum():
-    camera = skimage.img_as_ubyte(data.camera())
+    camera = util.img_as_ubyte(data.camera())
 
     threshold = threshold_minimum(camera)
-    assert_equal(threshold, 76)
+    assert_equal(threshold, 85)
 
-    astronaut = skimage.img_as_ubyte(data.astronaut())
+    astronaut = util.img_as_ubyte(data.astronaut())
     threshold = threshold_minimum(astronaut)
     assert_equal(threshold, 114)
 
+def test_threshold_minimum_histogram():
+    camera = util.img_as_ubyte(data.camera())
+    hist = histogram(camera.ravel(), 256, source_range='image')
+    threshold = threshold_minimum(hist=hist)
+    assert_equal(threshold, 85)
+
+def test_threshold_minimum_counts():
+    camera = util.img_as_ubyte(data.camera())
+    counts, bin_centers = histogram(camera.ravel(), 256, source_range='image')
+    threshold = threshold_minimum(hist=counts)
+    assert_equal(threshold, 85)
 
 def test_threshold_minimum_synthetic():
     img = np.arange(25*25, dtype=np.uint8).reshape((25, 25))
@@ -421,12 +512,12 @@ def test_triangle_float_images():
     int_bins = text.max() - text.min() + 1
     # Set nbins to match the uint case and threshold as float.
     assert(round(threshold_triangle(
-        text.astype(np.float), nbins=int_bins)) == 104)
+        text.astype(float), nbins=int_bins)) == 104)
     # Check that rescaling image to floats in unit interval is equivalent.
     assert(round(threshold_triangle(text / 255., nbins=int_bins) * 255) == 104)
     # Repeat for inverted image.
     assert(round(threshold_triangle(
-        np.invert(text).astype(np.float), nbins=int_bins)) == 151)
+        np.invert(text).astype(float), nbins=int_bins)) == 151)
     assert (round(threshold_triangle(
         np.invert(text) / 255., nbins=int_bins) * 255) == 151)
 
@@ -449,11 +540,18 @@ def test_triangle_flip():
     assert(len(unequal_pos[0]) / t_img.size < 1e-2)
 
 
-def test_mean_std_2d():
+@pytest.mark.parametrize(
+    "window_size, mean_kernel",
+    [(11, np.full((11,) * 2,  1 / 11 ** 2)),
+     ((11, 11), np.full((11, 11), 1 / 11 ** 2)),
+     ((9, 13), np.full((9, 13), 1 / np.prod((9, 13)))),
+     ((13, 9), np.full((13, 9), 1 / np.prod((13, 9)))),
+     ((1, 9), np.full((1, 9), 1 / np.prod((1, 9))))
+     ]
+)
+def test_mean_std_2d(window_size, mean_kernel):
     image = np.random.rand(256, 256)
-    window_size = 11
     m, s = _mean_std(image, w=window_size)
-    mean_kernel = np.ones((window_size,) * 2) / window_size**2
     expected_m = ndi.convolve(image, mean_kernel, mode='mirror')
     np.testing.assert_allclose(m, expected_m)
     expected_s = ndi.generic_filter(image, np.std, size=window_size,
@@ -461,10 +559,15 @@ def test_mean_std_2d():
     np.testing.assert_allclose(s, expected_s)
 
 
-def test_mean_std_3d():
+@pytest.mark.parametrize(
+    "window_size, mean_kernel", [
+        (5, np.full((5,) * 3, 1 / 5) ** 3),
+        ((5, 5, 5), np.full((5, 5, 5), 1 / 5 ** 3)),
+        ((1, 5, 5), np.full((1, 5, 5), 1 / 5 ** 2)),
+        ((3, 5, 7), np.full((3, 5, 7), 1 / np.prod((3, 5, 7))))]
+)
+def test_mean_std_3d(window_size, mean_kernel):
     image = np.random.rand(40, 40, 40)
-    window_size = 5
-    mean_kernel = np.ones((window_size,) * 3) / window_size**3
     m, s = _mean_std(image, w=window_size)
     expected_m = ndi.convolve(image, mean_kernel, mode='mirror')
     np.testing.assert_allclose(m, expected_m)
@@ -481,3 +584,85 @@ def test_niblack_sauvola_pathological_image():
     value = 0.03082192 + 2.19178082e-09
     src_img = np.full((4, 4), value).astype(np.float64)
     assert not np.any(np.isnan(threshold_niblack(src_img)))
+
+
+def test_bimodal_multiotsu_hist():
+    for name in ['camera', 'moon', 'coins', 'text', 'clock', 'page']:
+        img = getattr(data, name)()
+        assert threshold_otsu(img) == threshold_multiotsu(img, 2)
+
+    for name in ['chelsea', 'coffee', 'astronaut', 'rocket']:
+        img = rgb2gray(getattr(data, name)())
+        assert threshold_otsu(img) == threshold_multiotsu(img, 2)
+
+
+def test_check_multiotsu_results():
+    image = 0.25 * np.array([[0, 1, 2, 3, 4],
+                             [0, 1, 2, 3, 4],
+                             [0, 1, 2, 3, 4],
+                             [0, 1, 2, 3, 4]])
+
+    for idx in range(3, 6):
+        thr_multi = threshold_multiotsu(image,
+                                        classes=idx)
+        assert len(thr_multi) == idx-1
+
+
+def test_multiotsu_output():
+    image = np.zeros((100, 100), dtype='int')
+    coords = [(25, 25), (50, 50), (75, 75)]
+    values = [64, 128, 192]
+    for coor, val in zip(coords, values):
+        rr, cc = disk(coor, 20)
+        image[rr, cc] = val
+    thresholds = [0, 64, 128]
+    assert np.array_equal(thresholds, threshold_multiotsu(image, classes=4))
+
+
+def test_multiotsu_astro_image():
+    img = util.img_as_ubyte(data.astronaut())
+    with expected_warnings(['grayscale']):
+        assert_almost_equal(threshold_multiotsu(img), [58, 149])
+
+
+def test_multiotsu_more_classes_then_values():
+    img = np.ones((10, 10), dtype=np.uint8)
+    with testing.raises(ValueError):
+        threshold_multiotsu(img, classes=2)
+    img[:, 3:] = 2
+    with testing.raises(ValueError):
+        threshold_multiotsu(img, classes=3)
+    img[:, 6:] = 3
+    with testing.raises(ValueError):
+        threshold_multiotsu(img, classes=4)
+
+
+@pytest.mark.parametrize("thresholding, lower, upper", [
+    (threshold_otsu, 101, 103),
+    (threshold_yen, 145, 147),
+    (threshold_isodata, 101, 103),
+    (threshold_mean, 128, 130),
+    (threshold_triangle, 41, 43),
+    (threshold_minimum, 84, 86),
+])
+def test_thresholds_dask_compatibility(thresholding, lower, upper):
+    pytest.importorskip('dask', reason="dask python library is not installed")
+    import dask.array as da
+    dask_camera = da.from_array(data.camera(), chunks=(256, 256))
+    assert lower < float(thresholding(dask_camera)) < upper
+
+
+def test_multiotsu_lut():
+    for classes in [2, 3, 4]:
+        for name in ['camera', 'moon', 'coins', 'text', 'clock', 'page']:
+            img = getattr(data, name)()
+            prob, bin_centers = histogram(img.ravel(),
+                                          nbins=256,
+                                          source_range='image',
+                                          normalize=True)
+            prob = prob.astype('float32')
+
+            result_lut = _get_multiotsu_thresh_indices_lut(prob, classes - 1)
+            result = _get_multiotsu_thresh_indices(prob, classes - 1)
+
+            assert np.array_equal(result_lut, result)
