@@ -1,9 +1,17 @@
 from .spath import shortest_path
 from .mcp import MCP, MCP_Geometric, MCP_Connect, MCP_Flexible, route_through_array
 
+
 __all__ = ['shortest_path',
            'MCP',
            'MCP_Geometric',
            'MCP_Connect',
            'MCP_Flexible',
-           'route_through_array']
\ No newline at end of file
+           'route_through_array',
+           'rag_mean_color',
+           'cut_threshold',
+           'cut_normalized',
+           'ncut',
+           'draw_rag',
+           'merge_hierarchical',
+           'RAG']
