@@ -1,7 +1,7 @@
 import numpy as np
 
 from ..io.manage_plugins import call_plugin
-from ..color import rgb2gray
+from ..color.colorconv import rgb2gray, rgba2rgb
 from .util import file_or_url_context
 from ..exposure import is_low_contrast
 from .._shared.utils import warn
@@ -56,6 +56,8 @@ def imread(fname, as_gray=False, plugin=None, **plugin_args):
             img = np.swapaxes(img, -2, -3)
 
         if as_gray:
+            if img.shape[2] == 4:
+                img = rgba2rgb(img)
             img = rgb2gray(img)
 
     return img
@@ -81,7 +83,7 @@ def imread_collection(load_pattern, conserve_memory=True,
     ic : ImageCollection
         Collection of images.
 
-    Other parameters
+    Other Parameters
     ----------------
     plugin_args : keywords
         Passed to the given plugin.
@@ -108,7 +110,7 @@ def imsave(fname, arr, plugin=None, check_contrast=True, **plugin_args):
     check_contrast : bool, optional
         Check for low contrast and print warning (default: True).
 
-    Other parameters
+    Other Parameters
     ----------------
     plugin_args : keywords
         Passed to the given plugin.
@@ -124,10 +126,13 @@ def imsave(fname, arr, plugin=None, check_contrast=True, **plugin_args):
     if plugin is None and hasattr(fname, 'lower'):
         if fname.lower().endswith(('.tiff', '.tif')):
             plugin = 'tifffile'
+    if arr.dtype == bool:
+        warn('%s is a boolean image: setting True to 255 and False to 0. '
+             'To silence this warning, please convert the image using '
+             'img_as_ubyte.' % fname, stacklevel=2)
+        arr = arr.astype('uint8') * 255
     if check_contrast and is_low_contrast(arr):
         warn('%s is a low contrast image' % fname)
-    if arr.dtype == bool:
-        warn('%s is a boolean image: setting True to 1 and False to 0' % fname)
     return call_plugin('imsave', fname, arr, plugin=plugin, **plugin_args)
 
 
@@ -143,7 +148,7 @@ def imshow(arr, plugin=None, **plugin_args):
         tried (starting with imageio) until a suitable
         candidate is found.
 
-    Other parameters
+    Other Parameters
     ----------------
     plugin_args : keywords
         Passed to the given plugin.
@@ -165,7 +170,7 @@ def imshow_collection(ic, plugin=None, **plugin_args):
         Name of plugin to use.  By default, the different plugins are
         tried until a suitable candidate is found.
 
-    Other parameters
+    Other Parameters
     ----------------
     plugin_args : keywords
         Passed to the given plugin.
