@@ -43,7 +43,7 @@ class WindowManager(object):
         if self._gui_lock:
             raise GuiLockError(\
             'The gui lock can only be acquired by one toolkit per session. \
-            The lock is already aquired by %s' % self._guikit)
+            The lock is already acquired by %s' % self._guikit)
         else:
             self._gui_lock = True
             self._guikit = str(kit)
@@ -168,7 +168,6 @@ def histograms(image, nbins):
     -------
     out : (rcounts, gcounts, bcounts, vcounts)
         The binned histograms of the RGB channels and intensity values.
-
     This is a NAIVE histogram routine, meant primarily for fast display.
 
     '''
@@ -208,7 +207,7 @@ class ThreadDispatch(object):
             self.chunks.append((img[(3 * height // 4):, :, :],
                                 stateimg[(3 * height // 4):, :, :]))
 
-        # if they dont have 1, or 4 or more, 2 is good.
+        # if they don't have 1, or 4 or more, 2 is good.
         else:
             self.chunks.append((img[:(height // 2), :, :],
                                 stateimg[:(height // 2), :, :]))
@@ -296,7 +295,8 @@ class ColorMixer(object):
             can be positive or negative.
 
         '''
-        assert channel in self.valid_channels
+        if channel not in self.valid_channels:
+            raise ValueError('assert_channel is not a valid channel.')
         pool = ThreadDispatch(self.img, self.stateimg,
                               _colormixer.add, channel, ammount)
         pool.run()
@@ -304,7 +304,7 @@ class ColorMixer(object):
     def multiply(self, channel, ammount):
         '''Mutliply the indicated channel by the specified value.
 
-         Parameters
+        Parameters
         ----------
         channel : flag
             the color channel to operate on
@@ -314,7 +314,8 @@ class ColorMixer(object):
             can be positive or negative.
 
         '''
-        assert channel in self.valid_channels
+        if channel not in self.valid_channels:
+            raise ValueError('assert_channel is not a valid channel.')
         pool = ThreadDispatch(self.img, self.stateimg,
                               _colormixer.multiply, channel, ammount)
         pool.run()
@@ -328,7 +329,6 @@ class ColorMixer(object):
             The ammount to add to each channel.
         factor : float
             The factor to multiply each channel by.
-
         result = clip((pixel + offset)*factor)
 
         '''
