@@ -1,4 +1,3 @@
-from __future__ import division
 
 import numpy as np
 from skimage import io
@@ -20,15 +19,11 @@ im_hi = imf + 10
 
 imshow_expected_warnings = [
     r"tight_layout : falling back to Agg|\A\Z",
+    r"tight_layout: falling back to Agg|\A\Z",  # formatting change in mpl
     # Maptlotlib 2.2.3 seems to use np.asscalar which issues a warning
     # with numpy 1.16
     # Matplotlib 2.2.3 is the last supported version for python 2.7
-    r"np.asscalar|\A\Z",
-    # Warning raised by matplotlib on Windows:
-    # "The CObject type is marked Pending Deprecation in Python 2.7.
-    #  Please use capsule objects instead."
-    # Ref: https://docs.python.org/2/c-api/cobject.html
-    r"CObject type is marked|\A\Z"
+    r"np.asscalar|\A\Z"
 ]
 
 
@@ -56,7 +51,8 @@ def n_subplots(ax_im):
 
 def test_uint8():
     plt.figure()
-    with expected_warnings(imshow_expected_warnings):
+    with expected_warnings(imshow_expected_warnings +
+                           [r"CObject type is marked|\A\Z"]):
         ax_im = io.imshow(im8)
     assert ax_im.cmap.name == 'gray'
     assert ax_im.get_clim() == (0, 255)
@@ -66,7 +62,8 @@ def test_uint8():
 
 def test_uint16():
     plt.figure()
-    with expected_warnings(imshow_expected_warnings):
+    with expected_warnings(imshow_expected_warnings +
+                           [r"CObject type is marked|\A\Z"]):
         ax_im = io.imshow(im16)
     assert ax_im.cmap.name == 'gray'
     assert ax_im.get_clim() == (0, 65535)
@@ -76,7 +73,8 @@ def test_uint16():
 
 def test_float():
     plt.figure()
-    with expected_warnings(imshow_expected_warnings):
+    with expected_warnings(imshow_expected_warnings +
+                           [r"CObject type is marked|\A\Z"]):
         ax_im = io.imshow(imf)
     assert ax_im.cmap.name == 'gray'
     assert ax_im.get_clim() == (0, 1)
@@ -86,7 +84,7 @@ def test_float():
 
 def test_low_data_range():
     with expected_warnings(imshow_expected_warnings +
-                           [r"Low image data range|\A\Z"]):
+                           ["Low image data range|CObject type is marked"]):
         ax_im = io.imshow(im_lo)
     assert ax_im.get_clim() == (im_lo.min(), im_lo.max())
     # check that a colorbar was created
@@ -95,8 +93,12 @@ def test_low_data_range():
 
 def test_outside_standard_range():
     plt.figure()
+    # Warning raised by matplotlib on Windows:
+    # "The CObject type is marked Pending Deprecation in Python 2.7.
+    #  Please use capsule objects instead."
+    # Ref: https://docs.python.org/2/c-api/cobject.html
     with expected_warnings(imshow_expected_warnings +
-                           [r"out of standard range|\A\Z"]):
+                           ["out of standard range|CObject type is marked"]):
         ax_im = io.imshow(im_hi)
     assert ax_im.get_clim() == (im_hi.min(), im_hi.max())
     assert n_subplots(ax_im) == 2
@@ -105,8 +107,12 @@ def test_outside_standard_range():
 
 def test_nonstandard_type():
     plt.figure()
+    # Warning raised by matplotlib on Windows:
+    # "The CObject type is marked Pending Deprecation in Python 2.7.
+    #  Please use capsule objects instead."
+    # Ref: https://docs.python.org/2/c-api/cobject.html
     with expected_warnings(imshow_expected_warnings +
-                           ["Low image data range"]):
+                           ["Low image data range|CObject type is marked"]):
         ax_im = io.imshow(im64)
     assert ax_im.get_clim() == (im64.min(), im64.max())
     assert n_subplots(ax_im) == 2
@@ -117,7 +123,8 @@ def test_signed_image():
     plt.figure()
     im_signed = np.array([[-0.5, -0.2], [0.1, 0.4]])
 
-    with expected_warnings(imshow_expected_warnings):
+    with expected_warnings(imshow_expected_warnings +
+                           [r"CObject type is marked|\A\Z"]):
         ax_im = io.imshow(im_signed)
     assert ax_im.get_clim() == (-0.5, 0.5)
     assert n_subplots(ax_im) == 2
