@@ -1,13 +1,12 @@
-import os
 import itertools
 from tempfile import NamedTemporaryFile
-from ... import data_dir
-from .. import imread, imsave, use_plugin, reset_plugins
+from skimage.io import imread, imsave, use_plugin, reset_plugins
 import numpy as np
 
 from skimage._shared.testing import (assert_array_equal,
                                      assert_array_almost_equal,
-                                     parametrize)
+                                     parametrize, fetch)
+from skimage._shared import testing
 
 
 def setup():
@@ -20,32 +19,34 @@ def teardown():
 
 
 def test_imread_uint16():
-    expected = np.load(os.path.join(data_dir, 'chessboard_GRAY_U8.npy'))
-    img = imread(os.path.join(data_dir, 'chessboard_GRAY_U16.tif'))
+    expected = np.load(fetch('data/chessboard_GRAY_U8.npy'))
+    img = imread(fetch('data/chessboard_GRAY_U16.tif'))
     assert img.dtype == np.uint16
     assert_array_almost_equal(img, expected)
 
 
 def test_imread_uint16_big_endian():
-    expected = np.load(os.path.join(data_dir, 'chessboard_GRAY_U8.npy'))
-    img = imread(os.path.join(data_dir, 'chessboard_GRAY_U16B.tif'))
+    expected = np.load(fetch('data/chessboard_GRAY_U8.npy'))
+    img = imread(fetch('data/chessboard_GRAY_U16B.tif'))
     assert img.dtype == np.uint16
     assert_array_almost_equal(img, expected)
 
 
 def test_imread_multipage_rgb_tif():
-    img = imread(os.path.join(data_dir, 'multipage_rgb.tif'))
+    img = imread(fetch('data/multipage_rgb.tif'))
     assert img.shape == (2, 10, 10, 3), img.shape
 
+
 def test_tifffile_kwarg_passthrough ():
-    img = imread(os.path.join(data_dir, 'multipage.tif'), key=[1],
-                 multifile=False, multifile_close=True, fastij=True, 
+    img = imread(fetch('data/multipage.tif'), key=[1],
+                 multifile=False, multifile_close=True, fastij=True,
                  is_ome=True)
     assert img.shape == (15, 10), img.shape
 
+
 def test_imread_handle():
-    expected = np.load(os.path.join(data_dir, 'chessboard_GRAY_U8.npy'))
-    with open(os.path.join(data_dir, 'chessboard_GRAY_U16.tif'), 'rb') as fh:
+    expected = np.load(fetch('data/chessboard_GRAY_U8.npy'))
+    with open(fetch('data/chessboard_GRAY_U16.tif'), 'rb') as fh:
         img = imread(fh)
     assert img.dtype == np.uint16
     assert_array_almost_equal(img, expected)
@@ -56,7 +57,7 @@ class TestSave:
         f = NamedTemporaryFile(suffix='.tif')
         fname = f.name
         f.close()
-        imsave(fname, x)
+        imsave(fname, x, check_contrast=False)
         y = imread(fname)
         assert_array_equal(x, y)
 
