@@ -1,10 +1,9 @@
 from ._find_contours import find_contours
-from ._marching_cubes_lewiner import marching_cubes_lewiner
+from ._marching_cubes_lewiner import marching_cubes_lewiner, marching_cubes
 from ._marching_cubes_classic import (marching_cubes_classic,
                                       mesh_surface_area)
-from ._regionprops import regionprops, perimeter, regionprops_table
-from .simple_metrics import compare_mse, compare_nrmse, compare_psnr
-from ._structural_similarity import compare_ssim
+from ._regionprops import (regionprops, perimeter,
+                           perimeter_crofton, euler_number, regionprops_table)
 from ._polygon import approximate_polygon, subdivide_polygon
 from .pnpoly import points_in_poly, grid_points_in_poly
 from ._moments import (moments, moments_central, moments_coords,
@@ -21,6 +20,8 @@ __all__ = ['find_contours',
            'regionprops',
            'regionprops_table',
            'perimeter',
+           'perimeter_crofton',
+           'euler_number',
            'approximate_polygon',
            'subdivide_polygon',
            'LineModelND',
@@ -36,6 +37,7 @@ __all__ = ['find_contours',
            'moments_hu',
            'inertia_tensor',
            'inertia_tensor_eigvals',
+           'marching_cubes',
            'marching_cubes_lewiner',
            'marching_cubes_classic',
            'mesh_surface_area',
@@ -43,9 +45,5 @@ __all__ = ['find_contours',
            'label',
            'points_in_poly',
            'grid_points_in_poly',
-           'compare_ssim',
-           'compare_mse',
-           'compare_nrmse',
-           'compare_psnr',
            'shannon_entropy',
-]
+           ]
