@@ -10,12 +10,10 @@ Soille, P. (2003). Morphological Image Analysis: Principles and Applications
 (2nd ed.), Chapter 6. Springer-Verlag New York, Inc.
 """
 import numpy as np
-from scipy import ndimage as ndi
 
 from ..util import dtype_limits, invert, crop
 from .._shared.utils import warn
-from . import greyreconstruct
-from ._util import _offsets_to_raveled_neighbors
+from . import greyreconstruct, _util
 from ._extrema_cy import _local_maxima
 
 
@@ -55,10 +53,13 @@ def h_maxima(image, h, selem=None):
     neighborhood of the set.
 
     A local maximum M of height h is a local maximum for which
-    there is at least one path joining M with a higher maximum on which the
-    minimal value is f(M) - h (i.e. the values along the path are not
-    decreasing by more than h with respect to the maximum's value) and no
-    path for which the minimal value is greater.
+    there is at least one path joining M with an equal or higher local maximum
+    on which the minimal value is f(M) - h (i.e. the values along the path
+    are not decreasing by more than h with respect to the maximum's value)
+    and no path to an equal or higher local maximum for which the minimal
+    value is greater.
+
+    The global maxima of the image are also found by this function.
 
     Parameters
     ----------
@@ -74,9 +75,9 @@ def h_maxima(image, h, selem=None):
     Returns
     -------
     h_max : ndarray
-       The maxima of height >= h. The resulting image is a binary image, where
-       pixels belonging to the selected maxima take value 1, the others
-       take value 0.
+       The local maxima of height >= h and the global maxima.
+       The resulting image is a binary image, where pixels belonging to
+       the determined maxima take value 1, the others take value 0.
 
     See also
     --------
@@ -96,36 +97,80 @@ def h_maxima(image, h, selem=None):
 
     We create an image (quadratic function with a maximum in the center and
     4 additional constant maxima.
-    The heights of the maxima are: 1, 21, 41, 61, 81, 101
+    The heights of the maxima are: 1, 21, 41, 61, 81
 
     >>> w = 10
     >>> x, y = np.mgrid[0:w,0:w]
     >>> f = 20 - 0.2*((x - w/2)**2 + (y-w/2)**2)
     >>> f[2:4,2:4] = 40; f[2:4,7:9] = 60; f[7:9,2:4] = 80; f[7:9,7:9] = 100
-    >>> f = f.astype(np.int)
+    >>> f = f.astype(int)
 
     We can calculate all maxima with a height of at least 40:
 
     >>> maxima = extrema.h_maxima(f, 40)
 
-    The resulting image will contain 4 local maxima.
+    The resulting image will contain 3 local maxima.
     """
+
+    # Check for h value that is larger then range of the image. If this
+    # is True then there are no h-maxima in the image.
+    if h > np.ptp(image):
+        return np.zeros(image.shape, dtype=np.uint8)
+
+    # Check for floating point h value. For this to work properly
+    # we need to explicitly convert image to float64.
+    #
+    # FIXME: This could give incorrect results if image is int64 and
+    #        has a very high dynamic range. The dtype of image is
+    #        changed to float64, and different integer values could
+    #        become the same float due to rounding.
+    #
+    #   >>> ii64 = np.iinfo(np.int64)
+    #   >>> a = np.array([ii64.max, ii64.max - 2])
+    #   >>> a[0] == a[1]
+    #   False
+    #   >>> b = a.astype(np.float64)
+    #   >>> b[0] == b[1]
+    #   True
+    #
+    if np.issubdtype(type(h), np.floating) and \
+       np.issubdtype(image.dtype, np.integer):
+        if ((h % 1) != 0):
+            warn('possible precision loss converting image to '
+                 'floating point. To silence this warning, '
+                 'ensure image and h have same data type.',
+                 stacklevel=2)
+            image = image.astype(float)
+        else:
+            h = image.dtype.type(h)
+
+    if (h == 0):
+        raise ValueError("h = 0 is ambiguous, use local_maxima() "
+                         "instead?")
+
     if np.issubdtype(image.dtype, np.floating):
-        resolution = 2 * np.finfo(image.dtype).resolution
-        if h < resolution:
-            h = resolution
-        h_corrected = h - resolution / 2.0
-        shifted_img = image - h
+        # The purpose of the resolution variable is to allow for the
+        # small rounding errors that inevitably occur when doing
+        # floating point arithmetic. We want shifted_img to be
+        # guaranteed to be h less than image. If we only subtract h
+        # there may be pixels were shifted_img ends up being
+        # slightly greater than image - h.
+        #
+        # The resolution is scaled based on the pixel values in the
+        # image because floating point precision is relative. A
+        # very large value of 1.0e10 will have a large precision,
+        # say +-1.0e4, and a very small value of 1.0e-10 will have
+        # a very small precision, say +-1.0e-16.
+        #
+        resolution = 2 * np.finfo(image.dtype).resolution * np.abs(image)
+        shifted_img = image - h - resolution
     else:
         shifted_img = _subtract_constant_clip(image, h)
-        h_corrected = h
 
     rec_img = greyreconstruct.reconstruction(shifted_img, image,
                                              method='dilation', selem=selem)
     residue_img = image - rec_img
-    h_max = np.zeros(image.shape, dtype=np.uint8)
-    h_max[residue_img >= h_corrected] = 1
-    return h_max
+    return (residue_img >= h).astype(np.uint8)
 
 
 def h_minima(image, h, selem=None):
@@ -136,10 +181,13 @@ def h_minima(image, h, selem=None):
     neighborhood of the set.
 
     A local minimum M of depth h is a local minimum for which
-    there is at least one path joining M with a deeper minimum on which the
-    maximal value is f(M) + h (i.e. the values along the path are not
-    increasing by more than h with respect to the minimum's value) and no
-    path for which the maximal value is smaller.
+    there is at least one path joining M with an equal or lower local minimum
+    on which the maximal value is f(M) + h (i.e. the values along the path
+    are not increasing by more than h with respect to the minimum's value)
+    and no path to an equal or lower local minimum for which the maximal
+    value is smaller.
+
+    The global minima of the image are also found by this function.
 
     Parameters
     ----------
@@ -155,9 +203,9 @@ def h_minima(image, h, selem=None):
     Returns
     -------
     h_min : ndarray
-       The minima of depth >= h. The resulting image is a binary image, where
-       pixels belonging to the selected minima take value 1, the other pixels
-       take value 0.
+       The local minima of depth >= h and the global minima.
+       The resulting image is a binary image, where pixels belonging to
+       the determined minima take value 1, the others take value 0.
 
     See also
     --------
@@ -177,155 +225,48 @@ def h_minima(image, h, selem=None):
 
     We create an image (quadratic function with a minimum in the center and
     4 additional constant maxima.
-    The depth of the minima are: 1, 21, 41, 61, 81, 101
+    The depth of the minima are: 1, 21, 41, 61, 81
 
     >>> w = 10
     >>> x, y = np.mgrid[0:w,0:w]
     >>> f = 180 + 0.2*((x - w/2)**2 + (y-w/2)**2)
     >>> f[2:4,2:4] = 160; f[2:4,7:9] = 140; f[7:9,2:4] = 120; f[7:9,7:9] = 100
-    >>> f = f.astype(np.int)
+    >>> f = f.astype(int)
 
     We can calculate all minima with a depth of at least 40:
 
     >>> minima = extrema.h_minima(f, 40)
 
-    The resulting image will contain 4 local minima.
+    The resulting image will contain 3 local minima.
     """
+    if h > np.ptp(image):
+        return np.zeros(image.shape, dtype=np.uint8)
+
+    if np.issubdtype(type(h), np.floating) and \
+       np.issubdtype(image.dtype, np.integer):
+        if ((h % 1) != 0):
+            warn('possible precision loss converting image to '
+                 'floating point. To silence this warning, '
+                 'ensure image and h have same data type.',
+                 stacklevel=2)
+            image = image.astype(float)
+        else:
+            h = image.dtype.type(h)
+
+    if (h == 0):
+        raise ValueError("h = 0 is ambiguous, use local_minima() "
+                         "instead?")
+
     if np.issubdtype(image.dtype, np.floating):
-        resolution = 2 * np.finfo(image.dtype).resolution
-        if h < resolution:
-            h = resolution
-        h_corrected = h - resolution / 2.0
-        shifted_img = image + h
+        resolution = 2 * np.finfo(image.dtype).resolution * np.abs(image)
+        shifted_img = image + h + resolution
     else:
         shifted_img = _add_constant_clip(image, h)
-        h_corrected = h
 
     rec_img = greyreconstruct.reconstruction(shifted_img, image,
                                              method='erosion', selem=selem)
     residue_img = rec_img - image
-    h_min = np.zeros(image.shape, dtype=np.uint8)
-    h_min[residue_img >= h_corrected] = 1
-    return h_min
-
-
-def _set_edge_values_inplace(image, value):
-    """Set edge values along all axes to a constant value.
-
-    Parameters
-    ----------
-    image : ndarray
-        The array to modify inplace.
-    value : scalar
-        The value to use. Should be compatible with `image`'s dtype.
-
-    Examples
-    --------
-    >>> image = np.zeros((4, 5), dtype=int)
-    >>> _set_edge_values_inplace(image, 1)
-    >>> image
-    array([[1, 1, 1, 1, 1],
-           [1, 0, 0, 0, 1],
-           [1, 0, 0, 0, 1],
-           [1, 1, 1, 1, 1]])
-    """
-    for axis in range(image.ndim):
-        sl = [slice(None)] * image.ndim
-        # Set edge in front
-        sl[axis] = 0
-        image[tuple(sl)] = value
-        # Set edge to the end
-        sl[axis] = -1
-        image[tuple(sl)] = value
-
-
-def _fast_pad(image, value):
-    """Pad an array on all axes with one constant value.
-
-    Parameters
-    ----------
-    image : ndarray
-        Image to pad.
-    value : scalar
-         The value to use. Should be compatible with `image`'s dtype.
-
-    Returns
-    -------
-    padded_image : ndarray
-        The new image.
-
-    Notes
-    -----
-    The output of this function is equivalent to::
-
-        np.pad(image, mode="constant", constant_values=value)
-
-    However, this method needs to only allocate and copy once which can result
-    in significant speed gains if `image` is large.
-
-    Examples
-    --------
-    >>> _fast_pad(np.zeros((2, 3), dtype=int), 4)
-    array([[4, 4, 4, 4, 4],
-           [4, 0, 0, 0, 4],
-           [4, 0, 0, 0, 4],
-           [4, 4, 4, 4, 4]])
-    """
-    # Allocate padded image
-    new_shape = np.array(image.shape) + 2
-    new_image = np.empty(new_shape, dtype=image.dtype, order="C")
-
-    # Copy old image into new space
-    original_slice = tuple(slice(1, -1) for _ in range(image.ndim))
-    new_image[original_slice] = image
-    # and set the edge values
-    _set_edge_values_inplace(new_image, value)
-
-    return new_image
-
-
-def _resolve_neighborhood(selem, connectivity, ndim):
-    """Validate or create structuring element for use in `local_maxima`.
-
-    Depending on the values of `connectivity` and `selem` this function
-    either creates a new structuring element (`selem` is None) using
-    `connectivity` or validates the given structuring element (`selem` is not
-    None).
-
-    Parameters
-    ----------
-    selem : array-like or None
-        The structuring element to validate. See same argument in
-        `local_maxima`.
-    connectivity : int or None
-        A number used to determine the neighborhood of each evaluated pixel.
-        See same argument in `local_maxima`.
-    ndim : int
-        Number of dimensions `selem` ought to have.
-
-    Returns
-    -------
-    selem : ndarray
-        Validated or new structuring element specifying the neighborhood.
-    """
-    if selem is None:
-        if connectivity is None:
-            connectivity = ndim
-        selem = ndi.generate_binary_structure(ndim, connectivity)
-    else:
-        # Validate custom structured element
-        selem = np.asarray(selem, dtype=np.bool)
-        # Must specify neighbors for all dimensions
-        if selem.ndim != ndim:
-            raise ValueError(
-                "structuring element and image must have the same number of "
-                "dimensions"
-            )
-        # Must only specify direct neighbors
-        if any(s != 3 for s in selem.shape):
-            raise ValueError("dimension size in structuring element is not 3")
-
-    return selem
+    return (residue_img >= h).astype(np.uint8)
 
 
 def local_maxima(image, selem=None, connectivity=None, indices=False,
@@ -415,7 +356,7 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
     array([[False, False, False, False, False, False, False],
            [False,  True,  True, False, False, False, False],
            [False,  True,  True, False, False, False, False],
-           [ True, False, False, False, False, False,  True]], dtype=bool)
+           [ True, False, False, False, False, False,  True]])
     >>> local_maxima(image, indices=True)
     (array([1, 1, 2, 2, 3, 3]), array([1, 2, 1, 2, 0, 6]))
 
@@ -425,7 +366,7 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
     array([[False, False, False, False, False, False, False],
            [False,  True,  True, False,  True,  True, False],
            [False,  True,  True, False,  True,  True, False],
-           [ True, False, False, False, False, False,  True]], dtype=bool)
+           [ True, False, False, False, False, False,  True]])
 
     and exclude maxima that border the image edge:
 
@@ -433,7 +374,7 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
     array([[False, False, False, False, False, False, False],
            [False,  True,  True, False,  True,  True, False],
            [False,  True,  True, False,  True,  True, False],
-           [False, False, False, False, False, False, False]], dtype=bool)
+           [False, False, False, False, False, False, False]])
     """
     image = np.asarray(image, order="C")
     if image.size == 0:
@@ -442,17 +383,17 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
             # Make sure that output is a tuple of 1 empty array per dimension
             return np.nonzero(image)
         else:
-            return np.zeros(image.shape, dtype=np.bool)
+            return np.zeros(image.shape, dtype=bool)
 
     if allow_borders:
         # Ensure that local maxima are always at least one smaller sample away
         # from the image border
-        image = _fast_pad(image, image.min())
+        image = _util._fast_pad(image, image.min())
 
     # Array of flags used to store the state of each pixel during evaluation.
     # See _extrema_cy.pyx for their meaning
     flags = np.zeros(image.shape, dtype=np.uint8)
-    _set_edge_values_inplace(flags, value=3)
+    _util._set_border_values(flags, value=3)
 
     if any(s < 3 for s in image.shape):
         # Warn and skip if any dimension is smaller than 3
@@ -463,8 +404,8 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
             stacklevel=3
         )
     else:
-        selem = _resolve_neighborhood(selem, connectivity, image.ndim)
-        neighbor_offsets = _offsets_to_raveled_neighbors(
+        selem = _util._resolve_neighborhood(selem, connectivity, image.ndim)
+        neighbor_offsets = _util._offsets_to_raveled_neighbors(
             image.shape, selem, center=((1,) * image.ndim)
         )
 
@@ -483,12 +424,12 @@ def local_maxima(image, selem=None, connectivity=None, indices=False,
         flags = crop(flags, 1)
     else:
         # No padding was performed but set edge values back to 0
-        _set_edge_values_inplace(flags, value=0)
+        _util._set_border_values(flags, value=0)
 
     if indices:
         return np.nonzero(flags)
     else:
-        return flags.view(np.bool)
+        return flags.view(bool)
 
 
 def local_minima(image, selem=None, connectivity=None, indices=False,
@@ -572,7 +513,7 @@ def local_minima(image, selem=None, connectivity=None, indices=False,
     array([[False, False, False, False, False, False, False],
            [False,  True,  True, False, False, False, False],
            [False,  True,  True, False, False, False, False],
-           [ True, False, False, False, False, False,  True]], dtype=bool)
+           [ True, False, False, False, False, False,  True]])
     >>> local_minima(image, indices=True)
     (array([1, 1, 2, 2, 3, 3]), array([1, 2, 1, 2, 0, 6]))
 
@@ -582,7 +523,7 @@ def local_minima(image, selem=None, connectivity=None, indices=False,
     array([[False, False, False, False, False, False, False],
            [False,  True,  True, False,  True,  True, False],
            [False,  True,  True, False,  True,  True, False],
-           [ True, False, False, False, False, False,  True]], dtype=bool)
+           [ True, False, False, False, False, False,  True]])
 
     and exclude minima that border the image edge:
 
@@ -590,7 +531,7 @@ def local_minima(image, selem=None, connectivity=None, indices=False,
     array([[False, False, False, False, False, False, False],
            [False,  True,  True, False,  True,  True, False],
            [False,  True,  True, False,  True,  True, False],
-           [False, False, False, False, False, False, False]], dtype=bool)
+           [False, False, False, False, False, False, False]])
     """
     return local_maxima(
         image=invert(image),
