@@ -1,4 +1,3 @@
-import os
 import numpy as np
 from skimage.morphology import skeletonize, medial_axis, thin
 from skimage.morphology._skeletonize import (_generate_thin_luts,
@@ -6,10 +5,10 @@ from skimage.morphology._skeletonize import (_generate_thin_luts,
 from skimage import draw
 from scipy.ndimage import correlate
 from skimage.io import imread
-from skimage import data_dir
+from skimage import data
 
 from skimage._shared import testing
-from skimage._shared.testing import assert_array_equal
+from skimage._shared.testing import assert_array_equal, fetch
 
 
 class TestSkeletonize():
@@ -60,13 +59,13 @@ class TestSkeletonize():
         assert_array_equal(result, im)
 
     def test_skeletonize_output(self):
-        im = imread(os.path.join(data_dir, "bw_text.png"), as_gray=True)
+        im = imread(fetch("data/bw_text.png"), as_gray=True)
 
         # make black the foreground
         im = (im == 0)
         result = skeletonize(im)
 
-        expected = np.load(os.path.join(data_dir, "bw_text_skeleton.npy"))
+        expected = np.load(fetch("data/bw_text_skeleton.npy"))
         assert_array_equal(result, expected)
 
     def test_skeletonize_num_neighbours(self):
@@ -211,7 +210,7 @@ class TestMedialAxis():
                              [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0],
                              [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0],
                              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
-                            dtype=np.bool_)
+                            dtype=bool)
         result = medial_axis(image)
         assert np.all(result == expected)
         result, distance = medial_axis(image, return_distance=True)
@@ -231,7 +230,7 @@ class TestMedialAxis():
                              [0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0],
                              [0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0],
                              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
-                            dtype=np.bool_)
+                            dtype=bool)
         result = medial_axis(image)
         assert np.all(result == expected)
 
