@@ -1,30 +1,97 @@
-# coding: utf-8
 import scipy.stats
 import numpy as np
 from math import ceil
 from .. import img_as_float
-from ..restoration._denoise_cy import _denoise_bilateral, _denoise_tv_bregman
-from .._shared.utils import skimage_deprecation, warn
+from ._denoise_cy import _denoise_bilateral, _denoise_tv_bregman
+from .._shared.utils import warn
 import pywt
 import skimage.color as color
+from skimage.color.colorconv import ycbcr_from_rgb
 import numbers
 
 
+def _gaussian_weight(array, sigma_squared, *, dtype=float):
+    """Helping function. Define a Gaussian weighting from array and
+    sigma_square.
+
+    Parameters
+    ----------
+    array : ndarray
+        Input array.
+    sigma_squared : float
+        The squared standard deviation used in the filter.
+    dtype : data type object, optional (default : float)
+        The type and size of the data to be returned.
+
+    Returns
+    -------
+    gaussian : ndarray
+        The input array filtered by the Gaussian.
+    """
+    return np.exp(-0.5 * (array ** 2  / sigma_squared), dtype=dtype)
+
+
+def _compute_color_lut(bins, sigma, max_value, *, dtype=float):
+    """Helping function. Define a lookup table containing Gaussian filter
+    values using the color distance sigma.
+
+    Parameters
+    ----------
+     bins : int
+        Number of discrete values for Gaussian weights of color filtering.
+        A larger value results in improved accuracy.
+    sigma : float
+        Standard deviation for grayvalue/color distance (radiometric
+        similarity). A larger value results in averaging of pixels with larger
+        radiometric differences. Note, that the image will be converted using
+        the `img_as_float` function and thus the standard deviation is in
+        respect to the range ``[0, 1]``. If the value is ``None`` the standard
+        deviation of the ``image`` will be used.
+    max_value : float
+        Maximum value of the input image.
+    dtype : data type object, optional (default : float)
+        The type and size of the data to be returned.
+
+    Returns
+    -------
+    color_lut : ndarray
+        Lookup table for the color distance sigma.
+    """
+    values = np.linspace(0, max_value, bins, endpoint=False)
+    return _gaussian_weight(values, sigma**2, dtype=dtype)
+
+
+def _compute_spatial_lut(win_size, sigma, *, dtype=float):
+    """Helping function. Define a lookup table containing Gaussian filter
+    values using the spatial sigma.
+
+    Parameters
+    ----------
+    win_size : int
+        Window size for filtering.
+        If win_size is not specified, it is calculated as
+        ``max(5, 2 * ceil(3 * sigma_spatial) + 1)``.
+    sigma : float
+        Standard deviation for range distance. A larger value results in
+        averaging of pixels with larger spatial differences.
+    dtype : data type object
+        The type and size of the data to be returned.
+
+    Returns
+    -------
+    spatial_lut : ndarray
+        Lookup table for the spatial sigma.
+    """
+    grid_points = np.arange(-win_size // 2, win_size // 2 + 1)
+    rr, cc = np.meshgrid(grid_points, grid_points, indexing='ij')
+    distances = np.hypot(rr, cc)
+    return _gaussian_weight(distances, sigma**2, dtype=dtype).ravel()
+
+
 def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
-                      bins=10000, mode='constant', cval=0, multichannel=None):
+                      bins=10000, mode='constant', cval=0, multichannel=False):
     """Denoise image using bilateral filter.
 
-    This is an edge-preserving, denoising filter. It averages pixels based on
-    their spatial closeness and radiometric similarity [1]_.
-
-    Spatial closeness is measured by the Gaussian function of the Euclidean
-    distance between two pixels and a certain standard deviation
-    (`sigma_spatial`).
-
-    Radiometric similarity is measured by the Gaussian function of the
-    Euclidean distance between two color values and a certain standard
-    deviation (`sigma_color`).
-
     Parameters
     ----------
     image : ndarray, shape (M, N[, 3])
@@ -61,9 +128,24 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     denoised : ndarray
         Denoised image.
 
+    Notes
+    -----
+    This is an edge-preserving, denoising filter. It averages pixels based on
+    their spatial closeness and radiometric similarity [1]_.
+
+    Spatial closeness is measured by the Gaussian function of the Euclidean
+    distance between two pixels and a certain standard deviation
+    (`sigma_spatial`).
+
+    Radiometric similarity is measured by the Gaussian function of the
+    Euclidean distance between two color values and a certain standard
+    deviation (`sigma_color`).
+
     References
     ----------
-    .. [1] http://users.soe.ucsc.edu/~manduchi/Papers/ICCV98.pdf
+    .. [1] C. Tomasi and R. Manduchi. "Bilateral Filtering for Gray and Color
+           Images." IEEE International Conference on Computer Vision (1998)
+           839-846. :DOI:`10.1109/ICCV.1998.710815`
 
     Examples
     --------
@@ -72,12 +154,9 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     >>> astro = astro[220:300, 220:320]
     >>> noisy = astro + 0.6 * astro.std() * np.random.random(astro.shape)
     >>> noisy = np.clip(noisy, 0, 1)
-    >>> denoised = denoise_bilateral(noisy, sigma_color=0.05, sigma_spatial=15)
+    >>> denoised = denoise_bilateral(noisy, sigma_color=0.05, sigma_spatial=15,
+    ...                              multichannel=True)
     """
-    if multichannel is None:
-        warn('denoise_bilateral will default to multichannel=False in v0.15')
-        multichannel = True
-
     if multichannel:
         if image.ndim != 3:
             if image.ndim == 2:
@@ -113,11 +192,48 @@ def denoise_bilateral(image, win_size=None, sigma_color=None, sigma_spatial=1,
     if win_size is None:
         win_size = max(5, 2 * int(ceil(3 * sigma_spatial)) + 1)
 
-    return _denoise_bilateral(image, win_size, sigma_color, sigma_spatial,
-                              bins, mode, cval)
+    min_value = image.min()
+    max_value = image.max()
+
+    if min_value == max_value:
+        return image
+
+    # if image.max() is 0, then dist_scale can have an unverified value
+    # and color_lut[<int>(dist * dist_scale)] may cause a segmentation fault
+    # so we verify we have a positive image and that the max is not 0.0.
+    if min_value < 0.0:
+        raise ValueError("Image must contain only positive values")
+
+    if max_value == 0.0:
+        raise ValueError("The maximum value found in the image was 0.")
+
+    image = np.atleast_3d(img_as_float(image))
+    image = np.ascontiguousarray(image)
+
+    sigma_color = sigma_color or image.std()
+
+    color_lut = _compute_color_lut(bins, sigma_color, max_value,
+                                   dtype=image.dtype)
+
+    range_lut = _compute_spatial_lut(win_size, sigma_spatial, dtype=image.dtype)
+
+    out = np.empty(image.shape, dtype=image.dtype)
+
+    dims = image.shape[2]
+
+    # There are a number of arrays needed in the Cython function.
+    # It's easier to allocate them outside of Cython so that all
+    # arrays are in the same type, then just copy the empty array
+    # where needed within Cython.
+    empty_dims = np.empty(dims, dtype=image.dtype)
+
+    return _denoise_bilateral(image, image.max(), win_size, sigma_color,
+                              sigma_spatial, bins, mode, cval, color_lut,
+                              range_lut, empty_dims, out)
 
 
-def denoise_tv_bregman(image, weight, max_iter=100, eps=1e-3, isotropic=True):
+def denoise_tv_bregman(image, weight, max_iter=100, eps=1e-3, isotropic=True,
+                       *, multichannel=False):
     """Perform total-variation denoising using split-Bregman optimization.
 
     Total-variation denoising (also know as total-variation regularization)
@@ -143,6 +259,10 @@ def denoise_tv_bregman(image, weight, max_iter=100, eps=1e-3, isotropic=True):
         Maximal number of iterations used for the optimization.
     isotropic : boolean, optional
         Switch between isotropic and anisotropic TV denoising.
+    multichannel : bool, optional
+        Apply total-variation denoising separately for each channel. This
+        option should be true for color images, otherwise the denoising is
+        also applied in the channels dimension.
 
     Returns
     -------
@@ -151,17 +271,45 @@ def denoise_tv_bregman(image, weight, max_iter=100, eps=1e-3, isotropic=True):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Total_variation_denoising
+    .. [1] https://en.wikipedia.org/wiki/Total_variation_denoising
     .. [2] Tom Goldstein and Stanley Osher, "The Split Bregman Method For L1
            Regularized Problems",
            ftp://ftp.math.ucla.edu/pub/camreport/cam08-29.pdf
     .. [3] Pascal Getreuer, "Rudin–Osher–Fatemi Total Variation Denoising
            using Split Bregman" in Image Processing On Line on 2012–05–19,
-           http://www.ipol.im/pub/art/2012/g-tvd/article_lr.pdf
-    .. [4] http://www.math.ucsb.edu/~cgarcia/UGProjects/BregmanAlgorithms_JacquelineBush.pdf
+           https://www.ipol.im/pub/art/2012/g-tvd/article_lr.pdf
+    .. [4] https://web.math.ucsb.edu/~cgarcia/UGProjects/BregmanAlgorithms_JacquelineBush.pdf
 
     """
-    return _denoise_tv_bregman(image, weight, max_iter, eps, isotropic)
+    image = np.atleast_3d(img_as_float(image))
+
+    rows = image.shape[0]
+    cols = image.shape[1]
+    dims = image.shape[2]
+
+    shape_ext = (rows + 2, cols + 2, dims)
+
+    out = np.zeros(shape_ext, image.dtype)
+
+    if multichannel:
+        channel_out = np.zeros(shape_ext[:2] + (1,), dtype=out.dtype)
+        for c in range(image.shape[-1]):
+            # the algorithm below expects 3 dimensions to always be present.
+            # slicing the array in this fashion preserves the channel dimension for us
+            channel_in = np.ascontiguousarray(image[..., c:c+1])
+
+            _denoise_tv_bregman(channel_in, image.dtype.type(weight),
+                                max_iter, eps, isotropic, channel_out)
+
+            out[..., c] = channel_out[..., 0]
+
+    else:
+        image = np.ascontiguousarray(image)
+
+        _denoise_tv_bregman(image, image.dtype.type(weight), max_iter, eps,
+                            isotropic, out)
+
+    return np.squeeze(out[1:-1, 1:-1])
 
 
 def _denoise_tv_chambolle_nd(image, weight=0.1, eps=2.e-4, n_iter_max=200):
@@ -191,7 +339,6 @@ def _denoise_tv_chambolle_nd(image, weight=0.1, eps=2.e-4, n_iter_max=200):
     Notes
     -----
     Rudin, Osher and Fatemi algorithm.
-
     """
 
     ndim = image.ndim
@@ -282,7 +429,7 @@ def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, n_iter_max=200,
     Make sure to set the multichannel parameter appropriately for color images.
 
     The principle of total variation denoising is explained in
-    http://en.wikipedia.org/wiki/Total_variation_denoising
+    https://en.wikipedia.org/wiki/Total_variation_denoising
 
     The principle of total variation denoising is to minimize the
     total variation of the image, which can be roughly described as
@@ -312,7 +459,7 @@ def denoise_tv_chambolle(image, weight=0.1, eps=2.e-4, n_iter_max=200,
 
     >>> x, y, z = np.ogrid[0:20, 0:20, 0:20]
     >>> mask = (x - 22)**2 + (y - 20)**2 + (z - 17)**2 < 8**2
-    >>> mask = mask.astype(np.float)
+    >>> mask = mask.astype(float)
     >>> mask += 0.2*np.random.randn(*mask.shape)
     >>> res = denoise_tv_chambolle(mask, weight=100)
 
@@ -366,7 +513,7 @@ def _sigma_est_dwt(detail_coeffs, distribution='Gaussian'):
     ----------
     .. [1] D. L. Donoho and I. M. Johnstone. "Ideal spatial adaptation
        by wavelet shrinkage." Biometrika 81.3 (1994): 425-455.
-       DOI:10.1093/biomet/81.3.425
+       :DOI:`10.1093/biomet/81.3.425`
     """
     # Consider regions with detail coefficients exactly zero to be masked out
     detail_coeffs = detail_coeffs[np.nonzero(detail_coeffs)]
@@ -425,13 +572,16 @@ def _wavelet_threshold(image, wavelet, method=None, threshold=None,
     .. [1] Chang, S. Grace, Bin Yu, and Martin Vetterli. "Adaptive wavelet
            thresholding for image denoising and compression." Image Processing,
            IEEE Transactions on 9.9 (2000): 1532-1546.
-           DOI: 10.1109/83.862633
+           :DOI:`10.1109/83.862633`
     .. [2] D. L. Donoho and I. M. Johnstone. "Ideal spatial adaptation
            by wavelet shrinkage." Biometrika 81.3 (1994): 425-455.
-           DOI: 10.1093/biomet/81.3.425
-
+           :DOI:`10.1093/biomet/81.3.425`
     """
     wavelet = pywt.Wavelet(wavelet)
+    if not wavelet.orthogonal:
+        warn(("Wavelet thresholding was designed for use with orthogonal "
+              "wavelets. For nonorthogonal wavelets such as {}, results are "
+              "likely to be suboptimal.").format(wavelet.name))
 
     # original_extent is used to workaround PyWavelets issue #80
     # odd-sized input results in an image with 1 extra sample after waverecn
@@ -441,8 +591,7 @@ def _wavelet_threshold(image, wavelet, method=None, threshold=None,
     if wavelet_levels is None:
         # Determine the maximum number of possible levels for image
         dlen = wavelet.dec_len
-        wavelet_levels = np.min(
-            [pywt.dwt_max_level(s, dlen) for s in image.shape])
+        wavelet_levels = pywt.dwtn_max_level(image.shape, wavelet)
 
         # Skip coarsest wavelet scales (see Notes in docstring).
         wavelet_levels = max(wavelet_levels - 3, 1)
@@ -491,9 +640,61 @@ def _wavelet_threshold(image, wavelet, method=None, threshold=None,
     return pywt.waverecn(denoised_coeffs, wavelet)[original_extent]
 
 
+def _scale_sigma_and_image_consistently(image, sigma, multichannel,
+                                        rescale_sigma):
+    """If the ``image`` is rescaled, also rescale ``sigma`` consistently.
+
+    Images that are not floating point will be rescaled via ``img_as_float``.
+    """
+    if multichannel:
+        if isinstance(sigma, numbers.Number) or sigma is None:
+            sigma = [sigma] * image.shape[-1]
+        elif len(sigma) != image.shape[-1]:
+            raise ValueError(
+                "When multichannel is True, sigma must be a scalar or have "
+                "length equal to the number of channels")
+    if image.dtype.kind != 'f':
+        if rescale_sigma:
+            range_pre = image.max() - image.min()
+        image = img_as_float(image)
+        if rescale_sigma:
+            range_post = image.max() - image.min()
+            # apply the same magnitude scaling to sigma
+            scale_factor = range_post / range_pre
+            if multichannel:
+                sigma = [s * scale_factor if s is not None else s
+                         for s in sigma]
+            elif sigma is not None:
+                sigma *= scale_factor
+    return image, sigma
+
+
+def _rescale_sigma_rgb2ycbcr(sigmas):
+    """Convert user-provided noise standard deviations to YCbCr space.
+
+    Notes
+    -----
+    If R, G, B are linearly independent random variables and a1, a2, a3 are
+    scalars, then random variable C:
+        C = a1 * R + a2 * G + a3 * B
+    has variance, var_C, given by:
+        var_C = a1**2 * var_R + a2**2 * var_G + a3**2 * var_B
+    """
+    if sigmas[0] is None:
+        return sigmas
+    sigmas = np.asarray(sigmas)
+    rgv_variances = sigmas * sigmas
+    for i in range(3):
+        scalars = ycbcr_from_rgb[i, :]
+        var_channel = np.sum(scalars * scalars * rgv_variances)
+        sigmas[i] = np.sqrt(var_channel)
+    return sigmas
+
+
 def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
                     wavelet_levels=None, multichannel=False,
-                    convert2ycbcr=False, method='BayesShrink'):
+                    convert2ycbcr=False, method='BayesShrink',
+                    rescale_sigma=True):
     """Perform wavelet denoising on an image.
 
     Parameters
@@ -527,6 +728,13 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
     method : {'BayesShrink', 'VisuShrink'}, optional
         Thresholding method to be used. The currently supported methods are
         "BayesShrink" [1]_ and "VisuShrink" [2]_. Defaults to "BayesShrink".
+    rescale_sigma : bool, optional
+        If False, no rescaling of the user-provided ``sigma`` will be
+        performed. The default of ``True`` rescales sigma appropriately if the
+        image is rescaled internally.
+
+        .. versionadded:: 0.16
+           ``rescale_sigma`` was introduced in 0.16
 
     Returns
     -------
@@ -543,31 +751,45 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
     image, but larger thresholds also decrease the detail present in the image.
 
     If the input is 3D, this function performs wavelet denoising on each color
-    plane separately. The output image is clipped between either [-1, 1] and
-    [0, 1] depending on the input image range.
+    plane separately.
 
-    When YCbCr conversion is done, every color channel is scaled between 0
-    and 1, and `sigma` values are applied to these scaled color channels.
+    .. versionchanged:: 0.16
+       For floating point inputs, the original input range is maintained and
+       there is no clipping applied to the output. Other input types will be
+       converted to a floating point value in the range [-1, 1] or [0, 1]
+       depending on the input image range. Unless ``rescale_sigma = False``,
+       any internal rescaling applied to the ``image`` will also be applied
+       to ``sigma`` to maintain the same relative amplitude.
 
-    Many wavelet coefficient thresholding approaches have been proposed.  By
+    Many wavelet coefficient thresholding approaches have been proposed. By
     default, ``denoise_wavelet`` applies BayesShrink, which is an adaptive
     thresholding method that computes separate thresholds for each wavelet
     sub-band as described in [1]_.
 
     If ``method == "VisuShrink"``, a single "universal threshold" is applied to
-    all wavelet detail coefficients as described in [2]_.  This threshold
+    all wavelet detail coefficients as described in [2]_. This threshold
     is designed to remove all Gaussian noise at a given ``sigma`` with high
     probability, but tends to produce images that appear overly smooth.
 
+    Although any of the wavelets from ``PyWavelets`` can be selected, the
+    thresholding methods assume an orthogonal wavelet transform and may not
+    choose the threshold appropriately for biorthogonal wavelets. Orthogonal
+    wavelets are desirable because white noise in the input remains white noise
+    in the subbands. Biorthogonal wavelets lead to colored noise in the
+    subbands. Additionally, the orthogonal wavelets in PyWavelets are
+    orthonormal so that noise variance in the subbands remains identical to the
+    noise variance of the input. Example orthogonal wavelets are the Daubechies
+    (e.g. 'db2') or symmlet (e.g. 'sym2') families.
+
     References
     ----------
     .. [1] Chang, S. Grace, Bin Yu, and Martin Vetterli. "Adaptive wavelet
            thresholding for image denoising and compression." Image Processing,
            IEEE Transactions on 9.9 (2000): 1532-1546.
-           DOI: 10.1109/83.862633
+           :DOI:`10.1109/83.862633`
     .. [2] D. L. Donoho and I. M. Johnstone. "Ideal spatial adaptation
            by wavelet shrinkage." Biometrika 81.3 (1994): 425-455.
-           DOI: 10.1093/biomet/81.3.425
+           :DOI:`10.1093/biomet/81.3.425`
 
     Examples
     --------
@@ -576,7 +798,7 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
     >>> img = color.rgb2gray(img)
     >>> img += 0.1 * np.random.randn(*img.shape)
     >>> img = np.clip(img, 0, 1)
-    >>> denoised_img = denoise_wavelet(img, sigma=0.1)
+    >>> denoised_img = denoise_wavelet(img, sigma=0.1, rescale_sigma=True)
 
     """
     if method not in ["BayesShrink", "VisuShrink"]:
@@ -584,27 +806,43 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
             ('Invalid method: {}. The currently supported methods are '
              '"BayesShrink" and "VisuShrink"').format(method))
 
-    image = img_as_float(image)
+    # floating-point inputs are not rescaled, so don't clip their output.
+    clip_output = image.dtype.kind != 'f'
 
-    if multichannel:
-        if isinstance(sigma, numbers.Number) or sigma is None:
-            sigma = [sigma] * image.shape[-1]
+    if convert2ycbcr and not multichannel:
+        raise ValueError("convert2ycbcr requires multichannel == True")
 
+    image, sigma = _scale_sigma_and_image_consistently(image,
+                                                       sigma,
+                                                       multichannel,
+                                                       rescale_sigma)
     if multichannel:
         if convert2ycbcr:
             out = color.rgb2ycbcr(image)
+            # convert user-supplied sigmas to the new colorspace as well
+            if rescale_sigma:
+                sigma = _rescale_sigma_rgb2ycbcr(sigma)
             for i in range(3):
                 # renormalizing this color channel to live in [0, 1]
-                min, max = out[..., i].min(), out[..., i].max()
-                channel = out[..., i] - min
-                channel /= max - min
-                out[..., i] = denoise_wavelet(channel, wavelet=wavelet,
-                                              method=method, sigma=sigma[i],
+                _min, _max = out[..., i].min(), out[..., i].max()
+                scale_factor = _max - _min
+                if scale_factor == 0:
+                    # skip any channel containing only zeros!
+                    continue
+                channel = out[..., i] - _min
+                channel /= scale_factor
+                sigma_channel = sigma[i]
+                if sigma_channel is not None:
+                    sigma_channel /= scale_factor
+                out[..., i] = denoise_wavelet(channel,
+                                              wavelet=wavelet,
+                                              method=method,
+                                              sigma=sigma_channel,
                                               mode=mode,
-                                              wavelet_levels=wavelet_levels)
-
-                out[..., i] = out[..., i] * (max - min)
-                out[..., i] += min
+                                              wavelet_levels=wavelet_levels,
+                                              rescale_sigma=rescale_sigma)
+                out[..., i] = out[..., i] * scale_factor
+                out[..., i] += _min
             out = color.ycbcr2rgb(out)
         else:
             out = np.empty_like(image)
@@ -619,8 +857,10 @@ def denoise_wavelet(image, sigma=None, wavelet='db1', mode='soft',
                                  sigma=sigma, mode=mode,
                                  wavelet_levels=wavelet_levels)
 
-    clip_range = (-1, 1) if image.min() < 0 else (0, 1)
-    return np.clip(out, *clip_range)
+    if clip_output:
+        clip_range = (-1, 1) if image.min() < 0 else (0, 1)
+        out = np.clip(out, *clip_range, out=out)
+    return out
 
 
 def estimate_sigma(image, average_sigmas=False, multichannel=False):
@@ -655,7 +895,7 @@ def estimate_sigma(image, average_sigmas=False, multichannel=False):
     ----------
     .. [1] D. L. Donoho and I. M. Johnstone. "Ideal spatial adaptation
        by wavelet shrinkage." Biometrika 81.3 (1994): 425-455.
-       DOI:10.1093/biomet/81.3.425
+       :DOI:`10.1093/biomet/81.3.425`
 
     Examples
     --------
