@@ -1,4 +1,5 @@
 import numpy as np
+import re
 from skimage.transform._geometric import GeometricTransform
 from skimage.transform import (estimate_transform, matrix_transform,
                                EuclideanTransform, SimilarityTransform,
@@ -8,6 +9,7 @@ from skimage.transform import (estimate_transform, matrix_transform,
 
 from skimage._shared import testing
 from skimage._shared.testing import assert_equal, assert_almost_equal
+import textwrap
 
 
 SRC = np.array([
@@ -148,6 +150,19 @@ def test_similarity_init():
     assert_almost_equal(tform.rotation, rotation)
     assert_almost_equal(tform.translation, translation)
 
+    # test special case for scale where the rotation isn't exactly 90deg,
+    # but very close
+    scale = 1.0
+    rotation = np.pi / 2
+    translation = (0, 0)
+    params = np.array([[0, -1, 1.33226763e-15],
+                       [1, 2.22044605e-16, -1.33226763e-15],
+                       [0, 0, 1]])
+    tform = SimilarityTransform(params)
+    assert_almost_equal(tform.scale, scale)
+    assert_almost_equal(tform.rotation, rotation)
+    assert_almost_equal(tform.translation, translation)
+
 
 def test_affine_estimation():
     # exact solution
@@ -184,6 +199,9 @@ def test_affine_init():
     assert_almost_equal(tform2.shear, shear)
     assert_almost_equal(tform2.translation, translation)
 
+    # scalar vs. tuple scale arguments
+    assert_almost_equal(AffineTransform(scale=0.5).scale, AffineTransform(scale=(0.5, 0.5)).scale)
+
 
 def test_piecewise_affine():
     tform = PiecewiseAffineTransform()
@@ -381,6 +399,25 @@ def test_geometric_tform():
     with testing.raises(NotImplementedError):
         tform.__add__(0)
 
+    # See gh-3926 for discussion details
+    for i in range(20):
+        # Generate random Homography
+        H = np.random.rand(3, 3) * 100
+        H[2, H[2] == 0] += np.finfo(float).eps
+        H /= H[2, 2]
+
+        # Craft some src coords
+        src = np.array([
+            [(H[2, 1] + 1) / -H[2, 0], 1],
+            [1, (H[2, 0] + 1) / -H[2, 1]],
+            [1, 1],
+        ])
+        # Prior to gh-3926, under the above circumstances,
+        # destination coordinates could be returned with nan/inf values.
+        tform = ProjectiveTransform(H)  # Construct the transform
+        dst = tform(src)  # Obtain the dst coords
+        # Ensure dst coords are finite numeric values
+        assert(np.isfinite(dst).all())
 
 def test_invalid_input():
     with testing.raises(ValueError):
@@ -438,3 +475,49 @@ def test_degenerate():
     tform = ProjectiveTransform()
     tform.estimate(src, dst)
     assert np.all(np.isnan(tform.params))
+
+    # See gh-3926 for discussion details
+    tform = ProjectiveTransform()
+    for i in range(20):
+        # Some random coordinates
+        src = np.random.rand(4, 2) * 100
+        dst = np.random.rand(4, 2) * 100
+
+        # Degenerate the case by arranging points on a single line
+        src[:, 1] = np.random.rand()
+        # Prior to gh-3926, under the above circumstances,
+        # a transform could be returned with nan values.
+        assert(not tform.estimate(src, dst) or np.isfinite(tform.params).all())
+
+
+def test_projective_repr():
+    tform = ProjectiveTransform()
+    want = re.escape(textwrap.dedent(
+        '''
+        <ProjectiveTransform(matrix=
+            [[1., 0., 0.],
+             [0., 1., 0.],
+             [0., 0., 1.]]) at
+        ''').strip()) + ' 0x[a-f0-9]+' + re.escape('>')
+    # Hack the escaped regex to allow whitespace before each number for
+    # compatibility with different numpy versions.
+    want = want.replace('0\\.', ' *0\\.')
+    want = want.replace('1\\.', ' *1\\.')
+    assert re.match(want, repr(tform))
+
+
+def test_projective_str():
+    tform = ProjectiveTransform()
+    want = re.escape(textwrap.dedent(
+        '''
+        <ProjectiveTransform(matrix=
+            [[1., 0., 0.],
+             [0., 1., 0.],
+             [0., 0., 1.]])>
+        ''').strip())
+    # Hack the escaped regex to allow whitespace before each number for
+    # compatibility with different numpy versions.
+    want = want.replace('0\\.', ' *0\\.')
+    want = want.replace('1\\.', ' *1\\.')
+    print(want)
+    assert re.match(want, str(tform))
